@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -21,7 +21,7 @@
 #ifndef _SLRN_ART_H
 #define _SLRN_ART_H
 #ifndef SLRNPULL_CODE
-extern int slrn_select_article_mode (Slrn_Group_Type *, int, int);
+extern int slrn_select_article_mode (Slrn_Group_Type *, NNTP_Artnum_Type, int);
 extern void slrn_init_article_mode (void);
 extern void slrn_subject_strip_was (char *);
 extern SLKeyMap_List_Type *Slrn_Article_Keymap;
@@ -77,6 +77,7 @@ extern int Slrn_Sig_Is_End_Of_Article;
 extern int Slrn_Del_Article_Upon_Read;
 extern int Slrn_Followup_Strip_Sig;
 extern int Slrn_Smart_Quote;
+extern int Slrn_Pipe_Type;
 extern int Slrn_Process_Verbatim_Marks;
 #if SLRN_HAS_UUDEVIEW
 extern int Slrn_Use_Uudeview;
@@ -118,20 +119,20 @@ typedef struct Slrn_Header_Type
    struct Slrn_Header_Type *hash_next;  /* next in hash table */
    unsigned int num_children;
    unsigned long hash;		       /* based on msgid */
-   int number;			       /* server number */
+   NNTP_Artnum_Type number;			       /* server number */
    int lines;
    int bytes;
-   char *subject;		       /* malloced separately */
-   char *from;
+   char *subject;		       /* malloced */
+   char *from;			       /* malloced */
    char *date;			       /* malloced */
-   char *msgid;			       /* pointers to above space */
-   char *refs;
-   char *xref;
+   char *msgid;			       /* malloced */
+   char *refs;			       /* malloced */
+   char *xref;			       /* malloced */
    char *realname;		       /* malloced */
    unsigned int tag_number;
    Slrn_Header_Line_Type *add_hdrs;
 #define MAX_TREE_SIZE 256
-   char *tree_ptr;
+   char *tree_ptr;		       /* malloced -- could be NULL */
 #if SLRN_HAS_GROUPLENS
    int gl_rating;
    int gl_pred;
@@ -153,28 +154,40 @@ typedef struct Slrn_Article_Line_Type
 {
    struct Slrn_Article_Line_Type *next, *prev;
    unsigned int flags;
-#define LINE_TYPE_MASK		0x00FF
 #define HEADER_LINE		0x0001
 #define QUOTE_LINE		0x0002
 #define SIGNATURE_LINE		0x0004
 #define PGP_SIGNATURE_LINE	0x0008
 #define VERBATIM_LINE		0x0010
 #define VERBATIM_MARK_LINE	0x0020
+#define LINE_TYPE_MASK		0x00FF
 
-#define LINE_ATTRIBUTES_MASK	0x0700
 #define WRAPPED_LINE		0x0100
 #define SPOILER_LINE		0x0200
 #define HIDDEN_LINE		0x0400
+#define LINE_ATTRIBUTES_MASK	0x0700
 
-#define QUOTES_LEVEL_MASK	0x1000
-   /* The last few bits are devoted to quote levels. */
-#define QUOTE_LEVEL_SHIFT 12	       /* do NOT change this */
-#define QUOTE_LEVEL(flags) ((flags) >> QUOTE_LEVEL_SHIFT)
-
+#define LINE_HAS_8BIT_FLAG	0x8000
+   union
+     {
+	unsigned int quote_level;
+     }
+   v;
    char *buf;
 }
 Slrn_Article_Line_Type;
 
+typedef struct
+{
+    char *charset;		       /* malloced */
+    int content_type;
+    int content_subtype;
+    int was_modified;
+    int was_parsed;
+    int needs_metamail;
+}
+Slrn_Mime_Type;
+
 typedef struct
 {
    Slrn_Article_Line_Type *lines;
@@ -189,11 +202,7 @@ typedef struct
    int pgp_signature_hidden;
    int verbatim_hidden;
    int verbatim_marks_hidden;
-#if SLRN_HAS_MIME
-   int mime_was_modified;
-   int mime_was_parsed;
-   int mime_needs_metamail;
-#endif
+   Slrn_Mime_Type mime;
    int needs_sync;		       /* non-zero if line number/current line needs updated */
 }
 Slrn_Article_Type;
@@ -203,10 +212,8 @@ extern Slrn_Article_Type *Slrn_Current_Article;
 extern Slrn_Header_Type *slrn_find_header_with_msgid (char *);
 extern int slrn_set_visible_headers (char *);
 
-#if SLRN_HAS_SLANG
 extern char *Slrn_Visible_Headers_String;
 extern int slrn_is_hidden_headers_mode (void);
-#endif
 
 extern SLRegexp_Type *Slrn_Ignore_Quote_Regexp[];
 extern SLRegexp_Type *Slrn_Strip_Re_Regexp[];
@@ -260,8 +267,12 @@ extern int Slrn_Color_By_Score;
 extern int Slrn_Highlight_Unread;
 
 extern int slrn_set_header_format (unsigned int, char *);
+extern char *slrn_get_header_format (int num);   /* un-malloced */
 
 extern void slrn_art_sync_article (Slrn_Article_Type *);
+extern void slrn_art_free_line (Slrn_Article_Line_Type *);
+extern void slrn_art_free_article_line_list (Slrn_Article_Line_Type *);
+extern void slrn_art_free_article (Slrn_Article_Type *);
 extern int slrn_art_get_unread (void);
 
 /* These are in art_sort.c : */
