@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -44,6 +44,7 @@
 #include "slrn.h"
 #include "group.h"
 #include "util.h"
+#include "strutil.h"
 #include "server.h"
 #include "art.h"
 #include "misc.h"
@@ -54,7 +55,6 @@
 #include "score.h"
 #include "menu.h"
 #include "xover.h"
-#include "chmap.h"
 #include "print.h"
 
 /*}}}*/
@@ -130,13 +130,20 @@ static unsigned char *is_matching_line (unsigned char *b, SLRegexp_Type **r) /*{
    while (*r != NULL)
      {
 	SLRegexp_Type *re;
+	unsigned int match_len;
 
 	re = *r++;
+#if SLANG_VERSION < 20000
 	if ((re->min_length > len) 
 	    || (b != SLang_regexp_match (b, len, re)))
 	  continue;
-	
-	return b + re->end_matches[0];
+	match_len = re->end_matches[0];
+#else
+	if (b != (unsigned char *) SLregexp_match (re, (char *) b, len))
+	  continue;
+	(void) SLregexp_nth_match (re, 0, NULL, &match_len);
+#endif
+	return b + match_len;
      }
    return NULL;
 }
@@ -166,7 +173,7 @@ int _slrn_art_hide_quotes (Slrn_Article_Type *a, int reset) /*{{{*/
 	  {
 	     if (Art_Hide_Quote_Level > 1)
 	       {
-		  if (QUOTE_LEVEL(l->flags) >= (unsigned int) Art_Hide_Quote_Level - 1)
+		  if (l->v.quote_level + 1 >= (unsigned int) Art_Hide_Quote_Level)
 		    l->flags |= HIDDEN_LINE;
 	       }
 	     /* Show first line of quoted material; try to pick one that
@@ -216,7 +223,7 @@ void slrn_art_mark_spoilers (Slrn_Article_Type *a) /*{{{*/
    while ((l != NULL) && (l->flags & HEADER_LINE))
      l = l->next; /* skip header */
    
-   while ((l != NULL) && (NULL == strchr (l->buf, 12)))
+   while ((l != NULL) && (NULL == slrn_strbyte (l->buf, 12)))
      l = l->next; /* skip to first formfeed */
    
    while (l != NULL)
@@ -232,25 +239,37 @@ void slrn_art_mark_spoilers (Slrn_Article_Type *a) /*{{{*/
 static int try_supercite (Slrn_Article_Line_Type *l) /*{{{*/
 {
    Slrn_Article_Line_Type *last, *lsave;
+#if SLANG_VERSION < 20000
    static unsigned char compiled_pattern_buf[256];
-   static SLRegexp_Type re;
+   static SLRegexp_Type re_buf;
+   SLRegexp_Type *re;
+#else
+   static SLRegexp_Type *re = NULL;
+#endif
    unsigned char *b;
    int count;
    char name[32];
    unsigned int len;
+   unsigned int ofs;
    int ret;
 
-   re.pat = (unsigned char *) Super_Cite_Regexp;
-   re.buf = compiled_pattern_buf;
-   re.case_sensitive = 1;
-   re.buf_len = sizeof (compiled_pattern_buf);
+#if SLANG_VERSION < 20000
+   re = &re_buf;
+   re->pat = (unsigned char *) Super_Cite_Regexp;
+   re->buf = compiled_pattern_buf;
+   re->case_sensitive = 1;
+   re->buf_len = sizeof (compiled_pattern_buf);
+   if ((*compiled_pattern_buf == 0) && SLang_regexp_compile (re))
+     return -1;
+#else
+   if ((re == NULL)
+       && (NULL == (re = SLregexp_compile (Super_Cite_Regexp, 0))))
+     return -1;
+#endif
    
    /* skip header --- I should look for Xnewsreader: gnus */
    while ((l != NULL) && (*l->buf != 0)) l = l->next;
-   
-   if ((*compiled_pattern_buf == 0) && SLang_regexp_compile (&re))
-     return -1;
-   
+
    /* look at the first 15 lines on first attempt.
     * After that, scan the whole buffer looking for more citations */
    count = 15;
@@ -262,7 +281,7 @@ static int try_supercite (Slrn_Article_Line_Type *l) /*{{{*/
 	  {
 	     if ((l->flags & QUOTE_LINE) == 0)
 	       {
-		  if (NULL != slrn_regexp_match (&re, l->buf))
+		  if (NULL != slrn_regexp_match (re, l->buf))
 		    {
 		       l->flags |= QUOTE_LINE;
 		       break;
@@ -275,10 +294,12 @@ static int try_supercite (Slrn_Article_Line_Type *l) /*{{{*/
 	if ((l == NULL) || (count == 0)) return ret;
 	
 	/* Now find out what is used for citing. */
-	b = (unsigned char *) l->buf + re.beg_matches[1];
-	len = re.end_matches[1];
+	if (-1 == SLregexp_nth_match (re, 1, &ofs, &len))
+	  return ret;
+
+	b = (unsigned char *) l->buf + ofs;
 	if (len > sizeof (name) - 2) return ret;
-	
+
 	ret = 0;
 	strncpy (name, (char *) b, len); name[len] = 0;
 	
@@ -350,11 +371,10 @@ void slrn_art_mark_quotes (Slrn_Article_Type *a) /*{{{*/
 	  }
 	l->flags |= QUOTE_LINE;
 	level = 1;
-	while ((level < MAX_QUOTE_LEVELS) &&
-	       (NULL != (b = is_matching_line (b, Slrn_Ignore_Quote_Regexp))))
+	while (NULL != (b = is_matching_line (b, Slrn_Ignore_Quote_Regexp)))
 	  level++;
 
-	l->flags |= ((level-1) << QUOTE_LEVEL_SHIFT);
+	l->v.quote_level = level;
 	l = l->next;
      }
    a->is_modified = 1;
@@ -477,6 +497,10 @@ int _slrn_art_wrap_article (Slrn_Article_Type *a) /*{{{*/
 
    l = a->lines;
 
+#if SLANG_VERSION >= 20000
+   SLsmg_gotorc(0,0); /* used by Slsmg_strbytes later on */
+#endif
+
    while (l != NULL)
      {
 	unsigned char header_char_delimiter = 0;
@@ -489,11 +513,9 @@ int _slrn_art_wrap_article (Slrn_Article_Type *a) /*{{{*/
 		  continue;
 	       }
 
-	     if (0 == slrn_case_strncmp ((unsigned char *)"Path: ",
-					 (unsigned char *)l->buf, 6))
+	     if (0 == slrn_case_strncmp ("Path: ", l->buf, 6))
 	       header_char_delimiter = '!';
-	     else if (0 == slrn_case_strncmp ((unsigned char *) "Newsgroups: ",
-					      (unsigned char *)l->buf, 12))
+	     else if (0 == slrn_case_strncmp ( "Newsgroups: ", l->buf, 12))
 	       header_char_delimiter = ',';
 	  }
 	else if (l->flags & QUOTE_LINE)
@@ -510,21 +532,12 @@ int _slrn_art_wrap_article (Slrn_Article_Type *a) /*{{{*/
 	ch = *buf;
 	while (ch != 0)
 	  {
-	     if ((ch == '\t') && (SLsmg_Tab_Width > 0))
-	       {
-		  len += SLsmg_Tab_Width;
-		  len -= len % SLsmg_Tab_Width;
-	       }
-	     else if (((ch >= ' ') && (ch < 127))
-		      || (ch >= (unsigned char) SLsmg_Display_Eight_Bit))
-	       len++;
-	     else
-	       {
-		  len += 2;
-		  if (ch & 0x80) len++;
-	       }
-
-	     if (len > (unsigned int) SLtt_Screen_Cols)
+	     unsigned int bytes = SLsmg_strbytes (buf, buf+strlen((char *)buf),
+						  (unsigned int) SLtt_Screen_Cols);
+	     buf += bytes;
+	     while ((*buf == ' ') || (*buf == '\t'))
+	       buf++;
+	     if (*buf) /* we need to wrap */
 	       {
 		  Slrn_Article_Line_Type *new_l;
 		  unsigned char *buf0, *lbuf;
@@ -600,12 +613,18 @@ int _slrn_art_wrap_article (Slrn_Article_Type *a) /*{{{*/
 		  if (new_l->next != NULL) new_l->next->prev = new_l;
 
 		  new_l->flags = l->flags | WRAPPED_LINE;
+
+		  if (new_l->flags & QUOTE_LINE)
+		    new_l->v.quote_level = l->v.quote_level;
+
 		  l = new_l;
 		  buf = (unsigned char *) new_l->buf;
 		  len = 0;
 		  a->is_wrapped = 1;
 	       }
+#if SLANG_VERSION < 20000
 	     else buf++;
+#endif
 	     
 	     ch = *buf;
 	  }
@@ -755,8 +774,8 @@ char *slrn_art_extract_header (char *hdr, unsigned int len) /*{{{*/
    while ((l != NULL)
 	  && (*l->buf != 0))
      {
-	if (0 == slrn_case_strncmp ((unsigned char *) hdr,
-				    (unsigned char *) l->buf, len))
+	if (0 == slrn_case_strncmp ( hdr,
+				     l->buf, len))
 	  {
 	     char *result;
 
@@ -767,7 +786,7 @@ char *slrn_art_extract_header (char *hdr, unsigned int len) /*{{{*/
 	       }
 
 	     /* Return the data after the colon */
-	     result = slrn_strchr (l->buf, ':');
+	     result = slrn_strbyte (l->buf, ':');
 	     if (result == NULL) 
 	       result = l->buf + len;
 	     else result += 1;
@@ -793,9 +812,7 @@ Visible_Header_Type;
 
 static Visible_Header_Type *Visible_Headers;
 
-#if SLRN_HAS_SLANG
 char *Slrn_Visible_Headers_String;     /* for interpreter */
-#endif
 
 static void free_visible_header_list (void) /*{{{*/
 {
@@ -807,9 +824,8 @@ static void free_visible_header_list (void) /*{{{*/
 	SLfree ((char *) Visible_Headers);
 	Visible_Headers = next;
      }
-#if SLRN_HAS_SLANG
    SLang_free_slstring (Slrn_Visible_Headers_String);
-#endif
+   Slrn_Visible_Headers_String = NULL;
 }
 
 /*}}}*/
@@ -822,11 +838,9 @@ int slrn_set_visible_headers (char *headers) /*{{{*/
 
    free_visible_header_list ();
    
-#if SLRN_HAS_SLANG
    Slrn_Visible_Headers_String = SLang_create_slstring (headers);
    if (Slrn_Visible_Headers_String == NULL)
      return -1;
-#endif
 
    nth = 0;
    while (-1 != SLextract_list_element (headers, nth, ',', buf, sizeof(buf)))
@@ -887,8 +901,8 @@ void _slrn_art_hide_headers (Slrn_Article_Type *a) /*{{{*/
 	     char chv = (0x20 | v->header[hide]);
 	     
 	     if ((chv == ch)
-		 && (0 == slrn_case_strncmp ((unsigned char *)l->buf,
-					     (unsigned char *)v->header + hide,
+		 && (0 == slrn_case_strncmp (l->buf,
+					     v->header + hide,
 					     (v->len) - hide)))
 	       {
 		  hide_header = hide;
