@@ -3,8 +3,8 @@
  This file is part of SLRN.
  It contains the sorting routines for article mode.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -47,8 +47,10 @@
 #include "menu.h"
 #include "hash.h"
 #include "util.h"
+#include "strutil.h"
 #include "snprintf.h"
 #include "hooks.h"
+#include "common.h"
 
 /*}}}*/
 
@@ -639,12 +641,11 @@ static void link_same_subjects (void) /*{{{*/
 	  {
 	     int rslt;
 	     
-	     (void) slrn_run_hooks (HOOK_SUBJECT_COMPARE, 2, h->subject, h1->subject);
-	     
-	     if (-1 != SLang_pop_integer (&rslt))
+	     if ((1 == slrn_run_hooks (HOOK_SUBJECT_COMPARE, 2, h->subject, h1->subject))
+		 && (-1 != SLang_pop_integer (&rslt)))
 	       differ = rslt;
 	  }
-	
+
 	if (differ == 0)
 	  {
 	     /* h and h1 have the same subject. Now make h1 a (faked) child of h. */ 
@@ -794,8 +795,8 @@ static Slrn_Header_Type *fixup_thread_node (Slrn_Header_Type *h, char *tree) /*{
 	h->tree_ptr = NULL;
 	
 	if (*tree)
-	  h->tree_ptr = slrn_strmalloc (tree, 0);
-	
+	  h->tree_ptr = slrn_strmalloc (tree, 0);   /* NULL ok here */
+
 	h = h->sister;
 	last->next = h;
 	if (h == NULL) break;
@@ -911,7 +912,7 @@ static void sort_by_threads (void) /*{{{*/
 		  while (rparent->parent != NULL) rparent = rparent->parent;
 		  if (rparent == h) /* self referencing!!! */
 		    {
-		       slrn_error (_("Article %d is part of reference loop!"), h->number);
+		       slrn_error (_("Article " NNTP_FMT_ARTNUM " is part of reference loop!"), h->number);
 		    }
 		  else
 		    {
@@ -1037,23 +1038,23 @@ static void compile_function_list(char *order, sort_function_type **Functions) /
    
    while (-1 != SLextract_list_element (order, nth, ',', buf, sizeof(buf)))
      {
-	if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Subject"))
+	if (! slrn_case_strcmp(buf, "Subject"))
 	  add_sort_function(Functions, header_subject_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Score"))
+	else if (! slrn_case_strcmp(buf, "Score"))
 	  add_sort_function(Functions, header_score_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Highscore"))
+	else if (! slrn_case_strcmp(buf, "Highscore"))
 	  add_sort_function(Functions, header_highscore_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Date"))
+	else if (! slrn_case_strcmp(buf, "Date"))
 	  add_sort_function(Functions, header_date_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Author"))
+	else if (! slrn_case_strcmp(buf, "Author"))
 	  add_sort_function(Functions, header_author_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Lines"))
+	else if (! slrn_case_strcmp(buf, "Lines"))
 	  add_sort_function(Functions, header_lines_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Number"))
+	else if (! slrn_case_strcmp(buf, "Number"))
 	  add_sort_function(Functions, header_num_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Id"))
+	else if (! slrn_case_strcmp(buf, "Id"))
 	  add_sort_function(Functions, header_msgid_cmp, isupper(buf[0]));
-	else if (! slrn_case_strcmp((unsigned char*)buf, (unsigned char*)"Body"))
+	else if (! slrn_case_strcmp(buf, "Body"))
 	  add_sort_function(Functions, header_has_body_cmp, isupper(buf[0]));
 	else /* Nonexistant sorting method */
 	  {
@@ -1071,7 +1072,7 @@ static void recompile_sortorder(char *sort_order) /*{{{*/
 {
     char *separator;
 
-    separator=strchr(sort_order, '|');
+    separator = slrn_strbyte(sort_order, '|');
     if (separator) {
         *separator='\0';
         compile_function_list(sort_order, &Sort_Functions);
