@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -37,6 +37,7 @@
 #include "snprintf.h"
 #include "misc.h"
 #include "util.h"
+#include "strutil.h"
 
 static char *Global_Help [] = 
 {
@@ -47,9 +48,8 @@ static char *Global_Help [] =
    N_(" Questions not covered by the documentation can be posted to"),
    N_("   news.software.readers"),
    "",
-   N_(" Please email bug reports, suggestions or comments to the current maintainer,"),
-   N_("   Thomas Schultz <tststs@gmx.de>"),
-   N_(" The original author of this program is John E. Davis <davis@space.mit.edu>"),
+   N_(" Please email bug reports, suggestions or comments to the author of this"),
+   N_(" program, John E. Davis <jed@jedsoft.org>"),
    NULL
 };
 
@@ -222,7 +222,7 @@ static char *Group_Help [] =
 
 static char *Copyright_Notice [] =
 {
-   " Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>",
+   " Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>",
      "",
      " For parts of it:",
      " Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>",
@@ -418,15 +418,18 @@ int slrn_parse_helpfile (char *helpfile)
    char **current_help = NULL;
    int num_lines = 0;
    unsigned char *b;
+   int status;
 
    if (Slrn_Batch)
      return 0;
-   
+
    if (NULL == (fp = fopen (helpfile, "r"))) return -1;
-   while (fgets (buf, sizeof (buf) - 1, fp) != NULL)
+
+   status = -1;
+   while (NULL != fgets (buf, sizeof (buf) - 1, fp))
      {
 	ch = *buf;
-	
+
 	/* Skip over common comments */
 	if ((ch == '#') || (ch == '%') || (ch == ';') || (ch == '!'))
 	  continue;
@@ -466,19 +469,25 @@ int slrn_parse_helpfile (char *helpfile)
 	
 	slrn_free (current_help [num_lines]);
 	
-	if (NULL != (current_help [num_lines] = (char *) slrn_strmalloc (buf, 0)))
-	  num_lines++;
+	if (NULL == (current_help [num_lines] = (char *) slrn_strmalloc (buf, 0)))
+	  goto return_error;
+
+	num_lines++;
      }
+
+   status = 0;
+   /* drop */
+
+return_error:
    if (current_help != NULL)
      {
 	slrn_free (current_help[num_lines]);
 	current_help[num_lines] = NULL;
      }
    slrn_fclose (fp);
-   return 0;
+   return status;
 }
 
-
 void slrn_article_help (void)
 {
    char **h;
@@ -549,7 +558,7 @@ char *slrn_help_keyseq_from_function (char *f, SLKeyMap_List_Type *map) /*{{{*/
 
 /* In the following arrays, the same index corresponds to the same key */
 
-#define NUMBER_OF_KEYNAMES 28
+#define NUMBER_OF_KEYNAMES 37
 
 /* Symbolic names of keys as shown to the user */
 static char *KeyNames[NUMBER_OF_KEYNAMES] = /*{{{*/
@@ -568,6 +577,7 @@ static char *KeyNames[NUMBER_OF_KEYNAMES] = /*{{{*/
    N_( "<Enter>" ),
    N_( "<Return>" ),
    N_( "<Tab>" ),
+   N_( "<BackTab>" ),
    N_( "<Space>" ),
    N_( "<F1>" ),
    N_( "<F2>" ),
@@ -581,13 +591,22 @@ static char *KeyNames[NUMBER_OF_KEYNAMES] = /*{{{*/
    N_( "<F10>" ),
    N_( "<F11>" ),
    N_( "<F12>" ),
+   N_( "<F13>" ),
+   N_( "<F14>" ),
+   N_( "<F15>" ),
+   N_( "<F16>" ),
+   N_( "<F17>" ),
+   N_( "<F18>" ),
+   N_( "<F19>" ),
+   N_( "<F20>" ),
    N_( "<Esc>" )
 }; /*}}}*/
 
 /* Length information for the unlocalized versions */
 static unsigned char KeyNameLengths[NUMBER_OF_KEYNAMES] =
-{8,10,4,6,7,6,8,11,8,6,5,7,8,5,7,4,4,4,4,4,4,4,4,4,5,5,5,5};
+{8,10,4,6,7,6,8,11,8,6,5,7,8,5,9,7,4,4,4,4,4,4,4,4,4,5,5,5,5,5,5,5,5,5,5,5,5};
 
+#ifdef REAL_UNIX_SYSTEM
 /* Symbolic names of keys in the termcap database */
 static char *TermcapNames[NUMBER_OF_KEYNAMES] = /*{{{*/
 {
@@ -605,6 +624,7 @@ static char *TermcapNames[NUMBER_OF_KEYNAMES] = /*{{{*/
      "@8",
      "",
      "",
+     "kB",
      "",
      "k1",
      "k2",
@@ -618,8 +638,17 @@ static char *TermcapNames[NUMBER_OF_KEYNAMES] = /*{{{*/
      "k;",
      "F1",
      "F2",
+     "F3",
+     "F4",
+     "F5",
+     "F6",
+     "F7",
+     "F8",
+     "F9",
+     "FA",
      ""
 }; /*}}}*/
+#endif
 
 /* Default key sequences as fallbacks when termcap lookup has no result */
 static char *DefaultSequences[NUMBER_OF_KEYNAMES] = /*{{{*/
@@ -652,6 +681,7 @@ static char *DefaultSequences[NUMBER_OF_KEYNAMES] = /*{{{*/
      "\r",
      "\r",
      "\t",
+     "\033[Z",
      " ",
 #ifdef IBMPC_SYSTEM
      "^@;",
@@ -666,6 +696,14 @@ static char *DefaultSequences[NUMBER_OF_KEYNAMES] = /*{{{*/
      "^@D",
      "^@\x85",
      "^@\x86",
+     "^@T",
+     "^@U",
+     "^@V",
+     "^@W",
+     "^@X",
+     "^@Y",
+     "^@Z",
+     "^@[",
 #else /* NOT IBMPC_SYSTEM */
      "\033[11~",
      "\033[12~",
@@ -679,6 +717,14 @@ static char *DefaultSequences[NUMBER_OF_KEYNAMES] = /*{{{*/
      "\033[21~",
      "\033[23~",
      "\033[24~",
+     "\033[25~",
+     "\033[26~",
+     "\033[28~",
+     "\033[29~",
+     "\033[31~",
+     "\033[32~",
+     "\033[33~",
+     "\033[34~",
 #endif /* NOT IBMPC_SYSTEM */
      "\033"
 }; /*}}}*/
@@ -798,15 +844,15 @@ char *slrn_help_string_to_keyseq (char *s) /*{{{*/
    while (slen && (ind <= SLANG_MAX_KEYMAP_KEY_SEQ))
      {
 	char *end;
-	if ((*s == '<') && (NULL != (end = slrn_strchr (s,'>'))))
+	if ((*s == '<') && (NULL != (end = slrn_strbyte (s,'>'))))
 	  {
 	     int i;
 	     int len = end-s-1;
 	     for (i = 0; i < NUMBER_OF_KEYNAMES; i++)
 	       {
 		  if ((KeyNameLengths[i] == len+2) &&
-		      !slrn_case_strncmp ((unsigned char*)s+1,
-					  (unsigned char*)KeyNames[i]+1, len))
+		      !slrn_case_strncmp (s+1,
+					  KeyNames[i]+1, len))
 		    break;
 	       }
 	     if (i < NUMBER_OF_KEYNAMES)
