@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -24,9 +24,6 @@
 
 #include <stdio.h>
 
-/* rest of file inside this #if statement */
-#if SLRN_HAS_SLANG
-
 /*{{{ Include files */
 
 #if HAVE_STDLIB_H
@@ -52,18 +49,17 @@
 #include "print.h"
 #include "score.h"
 #include "snprintf.h"
-#include "mime.h"
 #include "version.h"
 #include "hooks.h"
 #include "help.h"
+#include "charset.h"
+#include "common.h"
+#include "strutil.h"
 
 /*}}}*/
 
 /*{{{ Public Global Variables */
 
-int Slrn_Use_Slang = 0;
-char *Slrn_Macro_Dir;
-
 /*}}}*/
 
 
@@ -110,7 +106,7 @@ static char **pop_argv_list (unsigned int *argcp)
       
    if (NULL == (argv = (char **) slrn_malloc (sizeof (char *) * (argc + 1), 1, 1)))
      {
-	SLang_Error = SL_MALLOC_ERROR;
+	SLang_set_error (SL_MALLOC_ERROR);
 	return NULL;
      }
    
@@ -218,7 +214,7 @@ static int get_yesno_cancel (char *prompt)
 
 static int get_response (char *choices, char *prompt)
 {
-   return slrn_get_response (choices, "%s", prompt);
+   return slrn_get_response (choices, choices, "%s", prompt);
 }
 
    
@@ -244,17 +240,9 @@ static void make_home_filename (char *name) /*{{{*/
 
 /*}}}*/
 
-static int evalfile (char *file)
-{
-   if (-1 == slrn_eval_slang_file (file))
-     return 0;
-
-   return 1;
-}
-
-
 int slrn_eval_slang_file (char *name) /*{{{*/
 {
+#if 0
    char file [SLRN_MAX_PATH_LEN];
    
    if (Slrn_Macro_Dir != NULL)
@@ -265,9 +253,9 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 	while (1)
 	  {
 	     if (-1 == SLextract_list_element (Slrn_Macro_Dir, n, ',',
-						   file, sizeof (file)))
+					       file, sizeof (file)))
 	       break;
-	     
+
 	     slrn_make_home_dirname (file, dir, sizeof (dir));
 	     if ((-1 != slrn_dircat (dir, name, file, sizeof (file)))
 		 && (1 == slrn_file_exists (file)))
@@ -283,11 +271,40 @@ int slrn_eval_slang_file (char *name) /*{{{*/
    
    slrn_make_home_filename (name, file, sizeof (file));
    slrn_message_now (_("loading %s"), file);
-   if (0 == SLang_load_file (file))
+   if (0 != SLang_load_file (file))
      return -1;
+   
+   return 0;
+#else
+
+   if (0 != SLang_load_file (name))
+     return -1;
+
+   return 0;
+#endif
+}
+
+int slrn_set_macro_dir (char *dir)
+{
+   if (-1 == SLang_run_hooks ("slrn_set_macro_dir_hook", 1, dir))
+     return -1;
+
    return 0;
 }
 
+char *slrn_get_macro_dir (void)
+{
+   char *str;
+
+   if (1 != SLang_run_hooks ("slrn_get_macro_dir_hook", 0))
+     return NULL;
+
+   if (-1 == SLpop_string (&str))
+     return NULL;
+   
+   return str;
+}
+     
 /*}}}*/
 
 
@@ -298,7 +315,12 @@ int slrn_eval_slang_file (char *name) /*{{{*/
 
 static void set_string_variable (char *s1, char *s2) /*{{{*/
 {
-   if (-1 == slrn_set_string_variable (s1, s2))
+   char *charset = Slrn_Display_Charset;
+
+   if (SLinterp_is_utf8_mode ())
+     charset = "utf-8";
+     
+   if (-1 == slrn_set_string_variable (s1, s2, charset))
      slrn_error (_("%s is not a valid variable name."), s1);
 }
 
@@ -314,29 +336,20 @@ static void set_integer_variable (char *s1, int *val) /*{{{*/
 
 static void get_variable_value (char *name) /*{{{*/
 {
-   char **s;
-   int *ip;
-   int type;
+   char *s;
+   int i;
+   SLtype type;
    
-   if (-1 == slrn_get_variable_value (name, &type, &s, &ip))
+   if (-1 == slrn_get_variable_value (name, &type, &s, &i))
+     return;
+
+   if (type == SLANG_STRING_TYPE)
      {
-	slrn_error (_("%s is not a valid variable name."), name);
+	(void) SLang_push_malloced_string (s);
 	return;
      }
-   
-   if (type == STRING_TYPE)
-     {
-	char *str;
-	if ((s == NULL) || (*s == NULL)) str = "";
-	else str = *s;
-	SLang_push_string (str);
-     }
-   else if (type == INT_TYPE)
-     {
-	int i;
-	if (ip == NULL) i = 0; else i = *ip;
-	SLang_push_integer (i);
-     }
+
+   (void) SLang_push_integer (i);
 }
 
 /*}}}*/
@@ -355,81 +368,8 @@ static char *interp_setlocale (int *category, char *locale) /*{{{*/
 
 /*}}}*/
 
-static void set_utf8_conversion_table (void) /*{{{*/
-{
-#if SLANG_VERSION < 10400
-   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
-#else
-# if ! SLRN_HAS_MIME
-   slrn_error (_("This version of slrn lacks MIME support."));
-# else
-   SLang_Array_Type *at;
-   int rows;
-   
-   if (-1 == SLang_pop_array_of_type (&at, SLANG_INT_TYPE))
-     {
-	slrn_error (_("Array of integer expected."));
-	return;
-     }
-   
-   if ((at->num_dims != 2) || (at->dims[0] != 2))
-     {
-	slrn_error (_("Two-dimensional array with two columns expected."));
-	goto free_and_return;
-     }
-   
-   slrn_free (Slrn_Utf8_Table);
-   Slrn_Utf8_Table = NULL;
-   
-   if ((rows = at->dims[1]) == 0)
-     goto free_and_return; /* no data, so set default (iso-8859-1) */
-   
-   if (NULL == (Slrn_Utf8_Table = slrn_malloc (65536, 1, 0)))
-     {
-	slrn_error (_("Could not get memory for conversion table."));
-	goto free_and_return;
-     }
-   
-   while (rows-- != 0)
-     {
-	int dims[2];
-	int unicode, local;
-	
-	dims[0] = 0;
-	dims[1] = rows;
-	(void) SLang_get_array_element (at, dims, &unicode);
-	
-	if (((unicode != 0) && (unicode < 128)) || (unicode > 65535))
-	  {
-	     slrn_error (_("Unicode value in row %d out of range."), rows);
-	     goto free_and_return;
-	  }
-	
-	dims[0] = 1;
-	(void) SLang_get_array_element (at, dims, &local);
-	
-	if ((local < 1) || (local > 255))
-	  {
-	     slrn_error (_("Local charset value in row %d out of range."),
-			 rows);
-	     goto free_and_return;
-	  }
-	
-	Slrn_Utf8_Table[unicode] = (char) local;
-     }
-   
-   free_and_return:
-   SLang_free_array (at);
-# endif /* SLRN_HAS_MIME */
-#endif /* SLANG_VERSION */
-}
-/*}}}*/
-
 static void generic_set_regexp (SLRegexp_Type **regexp_table) /*{{{*/
 {
-#if SLANG_VERSION < 10400
-   slrn_error (_("To use this feature, please update s-lang and recompile slrn."));
-#else
    SLang_Array_Type *at;
    SLcmd_Cmd_Table_Type cmd_table;
    int argc, i;
@@ -457,11 +397,8 @@ static void generic_set_regexp (SLRegexp_Type **regexp_table) /*{{{*/
    
    if (NULL == (cmd_table.string_args = (char**) slrn_malloc
 		((argc+1)*sizeof(char*), 0, 0)))
-     {
-	slrn_error (_("Failed to allocate memory."));
-	goto free_and_return;
-     }
-   
+     goto free_and_return;
+
    for (i = 0; i < argc; i++)
      {
 	(void) SLang_get_array_element (at, &i, &(cmd_table.string_args[i+1]));
@@ -473,7 +410,6 @@ static void generic_set_regexp (SLRegexp_Type **regexp_table) /*{{{*/
    
    free_and_return:
    SLang_free_array (at);
-#endif   
 }/*}}}*/
 
 static void set_ignore_quotes (void) /*{{{*/
@@ -531,17 +467,11 @@ static void definekey (char *fun, char *key, char *map) /*{{{*/
 	slrn_error (_("definekey: no such keymap."));
 	return;
      }
-   
-   if ((kmap == Slrn_Keymap_RLI->keymap) &&
-       (NULL == SLang_find_key_function(fun, kmap)))
-     {
-	SLKeymap_Function_Type *tmp = kmap->functions;
-	kmap->functions = Slrn_Custom_Readline_Functions;
-	SLang_define_key (convkey, fun, kmap);
-	kmap->functions = tmp;
-     }
+
+   if (kmap == SLrline_get_keymap (Slrn_Keymap_RLI))
+     (void) slrn_rline_setkey (convkey, fun, kmap);
    else
-     SLang_define_key (convkey, fun, kmap);
+     (void) SLang_define_key (convkey, fun, kmap);
 }
 
 /*}}}*/
@@ -631,12 +561,22 @@ static void read_mini_variable (char *prompt, char *dfl, char *init) /*{{{*/
 
 static int read_mini_integer (char *prompt, int *dfl) /*{{{*/
 {
-   int status, retval;
-   status = slrn_read_integer (prompt, dfl, &retval);
-   if (-1 == status)
-     error (_("Quit!"));
+   NNTP_Artnum_Type x;
+   int ix;
+
+   x = *dfl;
+   if (-1 == slrn_read_artnum_int (prompt, &x, &x))
+     {
+	error (_("Quit!"));
+	return -1;
+     }
+   
+   ix = (int) x;
+   if (x != ix)
+     error (_("Number exceeds the size of an integer"));
+
    slrn_clear_message ();
-   return retval;
+   return ix;
 }
 /*}}}*/
 
@@ -901,7 +841,7 @@ static char *generic_article_as_string (int use_orig) /*{{{*/
    
    if (NULL == (s = (char *) SLMALLOC (len + 1)))
      {
-	SLang_Error = SL_MALLOC_ERROR;
+	SLang_set_error (SL_MALLOC_ERROR);
 	return NULL;
      }
    
@@ -1209,18 +1149,21 @@ static int re_header_search (char *pat, unsigned int offset, int dir) /*{{{*/
 {
    SLRegexp_Type *re;
    Slrn_Header_Type *h = Slrn_Current_Header;
-   
+   int status;
+
    if ((-1 == check_article_mode ())
        || (h == NULL)
        || (NULL == (re = slrn_compile_regexp_pattern (pat))))
      return 0;
    
+   status = 0;
    while (h != NULL)
      {
 	if (NULL != slrn_regexp_match (re, *(char **) ((char *)h + offset)))
 	  {
 	     slrn_goto_header (h, 0);
-	     return 1;
+	     status = 1;
+	     break;
 	  }
 	
 	if (dir > 0)
@@ -1228,7 +1171,8 @@ static int re_header_search (char *pat, unsigned int offset, int dir) /*{{{*/
 	else
 	  h = h->prev;
      }
-   return 0;   
+   SLregexp_free (re);
+   return status;   
 }
 
 /*}}}*/
@@ -1410,8 +1354,12 @@ static void log_message (char *buf)
    FILE *fp;
    
    fp = Log_File_Ptr;
-   if (fp == NULL) fp = stderr;
-   
+   if (fp == NULL) 
+     {
+	if (Slrn_TT_Initialized & SLRN_SMG_INIT)
+	  return;
+	fp = stderr;
+     }
    fputs (buf, fp);
    fflush (fp);
 }
@@ -1492,6 +1440,41 @@ static void reload_scorefile (int *apply_now) /*{{{*/
 
 /*}}}*/
 
+static int do_register_unregister_hook (int reg)
+{
+   SLang_Name_Type *func;
+   char *name;
+   int status;
+
+   if (NULL == (func = SLang_pop_function ()))
+     return -1;
+   
+   if (-1 == SLang_pop_slstring (&name))
+     {
+	SLang_free_function (func);
+	return -1;
+     }
+   
+   if (reg)
+     status = slrn_register_hook (name, func);
+   else
+     status = slrn_unregister_hook (name, func);
+   
+   SLang_free_function (func);
+   SLang_free_slstring (name);
+   return status;
+}
+
+static int register_hook (void)
+{
+   return do_register_unregister_hook (1);
+}
+
+static int unregister_hook (void)
+{
+   return do_register_unregister_hook (0);
+}
+
 static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
 {
    MAKE_INTRINSIC_0("headers_hidden_mode", slrn_is_hidden_headers_mode, SLANG_INT_TYPE),
@@ -1574,7 +1557,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_SSS("read_mini_filename", read_mini_filename, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SSS("read_mini_variable", read_mini_variable, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SI("read_mini_integer", read_mini_integer, SLANG_INT_TYPE),
-   MAKE_INTRINSIC_SS("register_hook", slrn_register_hook_by_name, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("register_hook", register_hook, SLANG_INT_TYPE),
    MAKE_INTRINSIC_I("reload_scorefile", reload_scorefile, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("request_body", request_body, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_S("save_current_article", save_current_article, SLANG_INT_TYPE),
@@ -1601,7 +1584,6 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("set_strip_re_regexp", set_strip_re_regexp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_strip_sig_regexp", set_strip_sig_regexp, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_strip_was_regexp", set_strip_was_regexp, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_0("set_utf8_conversion_table", set_utf8_conversion_table, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("get_group_order", slrn_intr_get_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0("set_group_order", slrn_intr_set_group_order, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_IS("setlocale", interp_setlocale, SLANG_STRING_TYPE),
@@ -1612,7 +1594,7 @@ static SLang_Intrin_Fun_Type Slrn_Intrinsics [] = /*{{{*/
    MAKE_INTRINSIC_0("uncollapse_threads", uncollapse_threads, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_SS("undefinekey", undefinekey, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_I("ungetkey", ungetkey, SLANG_VOID_TYPE),
-   MAKE_INTRINSIC_SS("unregister_hook", slrn_unregister_hook_by_name, SLANG_INT_TYPE),
+   MAKE_INTRINSIC_0("unregister_hook", unregister_hook, SLANG_INT_TYPE),
    MAKE_INTRINSIC_0("update", update, SLANG_VOID_TYPE),
    MAKE_INTRINSIC_0(NULL, NULL, 0)
 };
@@ -1632,10 +1614,8 @@ static SLang_Intrin_Var_Type Intrin_Vars [] = /*{{{*/
    MAKE_VARIABLE("HEADER_LOW_SCORE", &Interp_Header_Low_Score, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("HEADER_READ", &Interp_Header_Read, SLANG_INT_TYPE, 1),
    MAKE_VARIABLE("HEADER_TAGGED", &Interp_Header_Tagged, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("SCREEN_HEIGHT", &SLtt_Screen_Rows, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("SCREEN_WIDTH", &SLtt_Screen_Cols, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version", &Slrn_Version_Number, SLANG_INT_TYPE, 1),
-   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version, SLANG_STRING_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version", &Slrn_Version, SLANG_INT_TYPE, 1),
+   MAKE_VARIABLE("_slrn_version_string", &Slrn_Version_String, SLANG_STRING_TYPE, 1),
 #ifdef HAVE_SETLOCALE
 # ifdef LC_CTYPE
    MAKE_VARIABLE("LC_CTYPE", &Interp_Lc_Ctype, SLANG_INT_TYPE, 1),
@@ -1657,36 +1637,79 @@ static int interp_system (char *s) /*{{{*/
 
 /*}}}*/
 
+static int add_intrinsic_variables (void)
+{
+   /* This is necessary because of the windows DLL limitations */
+   if ((-1 == SLadd_intrinsic_variable ("SCREEN_HEIGHT", (VOID_STAR)&SLtt_Screen_Rows, SLANG_INT_TYPE, 1))
+       || ((-1 == SLadd_intrinsic_variable ("SCREEN_WIDTH", (VOID_STAR)&SLtt_Screen_Cols, SLANG_INT_TYPE, 1)))
+      )
+     return -1;
+
+   return 0;
+}
+
+static int load_startup_file (void)
+{
+   char *dir;
+
+   if (NULL == (dir = getenv (ENV_SLRN_SLANG_DIR)))
+     dir = SLRN_SLANG_DIR;
+   
+   if ((dir == NULL) || (*dir == 0))
+     {
+	slrn_error (_("The SLRN_SLANG_DIR variable is either NULL or empty"));
+	return -1;
+     }
+   
+   if (-1 == SLpath_set_load_path (dir))
+     return -1;
+   
+   if (-1 == SLang_load_file ("slrn.sl"))
+     {
+	slrn_error (_("Configuration error: Unable to load startup file"));
+
+	if (SLang_get_error () == SL_Open_Error)
+	  {
+	     slrn_error (_("The hardcoded load path is set to: %s"), SLRN_SLANG_DIR);
+	     slrn_error (_("You make need to set the %s environment variable."), ENV_SLRN_SLANG_DIR);
+	  }
+	return -1;
+     }
+
+   return 0;
+}
+
 int slrn_init_slang (void) /*{{{*/
 {
-   Slrn_Use_Slang = 0;
    if ((-1 == SLang_init_slang ())
        || (-1 == SLang_init_slmath ())
+       || (-1 == SLang_init_array ())
        || (-1 == SLang_init_posix_process ())
        || (-1 == SLang_init_posix_dir ())
        || (-1 == SLang_init_stdio ())
        || (-1 == SLang_init_posix_io ())
        || (-1 == SLang_init_ospath ())
-       || (-1 == SLang_init_slassoc ())
-#if SLANG_VERSION >= 10400
        || (-1 == SLang_init_import ()) /* enable dynamic linking */
-#endif
-       
        /* Now add intrinsics for this application */
        || (-1 == SLadd_intrin_fun_table(Slrn_Intrinsics, NULL))
-       || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL)))
+       || (-1 == SLadd_intrin_var_table(Intrin_Vars, NULL))
+       || (-1 == add_intrinsic_variables ()))
      return -1;
-   
+
    SLadd_intrinsic_function ("system", (FVOID_STAR) interp_system, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
-   SLadd_intrinsic_function ("evalfile", (FVOID_STAR) evalfile, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE);
+   /* SLadd_intrinsic_function ("evalfile", (FVOID_STAR) evalfile, SLANG_INT_TYPE, 1, SLANG_STRING_TYPE); */
 
+   (void) SLang_load_file_verbose (1);
+   
    SLang_Error_Hook = error;
-
-   Slrn_Use_Slang = 1;
    SLang_User_Clear_Error = slrn_clear_message;
    SLang_Exit_Error_Hook = slrn_va_exit_error;
    SLang_Dump_Routine = log_message;
    SLang_VMessage_Hook = slrn_va_message;
+   
+   if (-1 == load_startup_file ())
+     return -1;
+
    return 0;
 }
 
@@ -1697,5 +1720,3 @@ int slrn_reset_slang (void)
    close_log_file ();
    return 0;
 }
-
-#endif
