@@ -1,7 +1,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -19,6 +19,6 @@
 */
 extern int slrn_init_slang (void);
 extern int slrn_eval_slang_file (char *);
-extern int Slrn_Use_Slang;
-extern char *Slrn_Macro_Dir;
+extern int slrn_set_macro_dir (char *dir);
+extern char *slrn_get_macro_dir (void);
 extern int slrn_reset_slang (void);
