@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001, 2002 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -45,6 +45,8 @@
 #include "util.h"
 #include "slrndir.h"
 #include "snprintf.h"
+#include "common.h"
+#include "strutil.h"
 
 /*}}}*/
 
@@ -108,12 +110,12 @@ static void update_menu (Menu_Type *m) /*{{{*/
    
    SLsmg_erase_eol ();
 
-   col = SLtt_Screen_Cols - 16;
+   col = SLtt_Screen_Cols - (5 + strlen(Slrn_Version_String));
    if (SLsmg_get_column () < col)
      SLsmg_gotorc (0, col);
    
    SLsmg_write_string ("slrn ");
-   SLsmg_write_string (Slrn_Version);
+   SLsmg_write_string (Slrn_Version_String);
    
    slrn_set_color (0);
 }
@@ -230,6 +232,42 @@ static void center_string_column (char *title, int row, int col, int num_columns
 
 /*}}}*/
 
+static void draw_box (int r0, int c0, unsigned int dr, unsigned int dc)
+{
+   unsigned int i;
+   int r1, c1;
+
+   if (Slrn_Simulate_Graphic_Chars == 0)
+     {
+	SLsmg_draw_box (r0, c0, dr, dc);
+	return;
+     }
+   
+   if ((dr == 0) || (dc == 0))
+     return;
+   dr--; dc--;
+
+   r1 = r0 + (int) dr;
+   c1 = c0 + (int) dc;
+
+   SLsmg_gotorc (r0, c0);
+   for (i = 0; i < dc; i++)
+     SLsmg_write_char ('-');
+   SLsmg_gotorc (r0+dr, c0);
+   for (i = 0; i < dc; i++)
+     SLsmg_write_char ('-');
+   for (i = 1; i < dr; i++)
+     {
+	SLsmg_gotorc (r0+i, c0);  SLsmg_write_char ('|');
+	SLsmg_gotorc (r0+i, c1);  SLsmg_write_char ('|');
+     }
+   SLsmg_gotorc (r0, c0);  SLsmg_write_char ('+');
+   SLsmg_gotorc (r0, c1);  SLsmg_write_char ('+');
+   SLsmg_gotorc (r1, c0);  SLsmg_write_char ('+');
+   SLsmg_gotorc (r1, c1);  SLsmg_write_char ('+');
+   SLsmg_gotorc (r0, c0);
+}
+
 static int draw_select_box (Slrn_Select_Box_Type *sb) /*{{{*/
 {
    int num_selections, max_selection_len;
@@ -297,7 +335,7 @@ static int draw_select_box (Slrn_Select_Box_Type *sb) /*{{{*/
    center_string_column (_("(Select One)"), r, column, num_columns);
 				      
    slrn_set_color (FRAME_COLOR);
-   SLsmg_draw_box (row, column, num_rows, num_columns);
+   draw_box (row, column, num_rows, num_columns);
    slrn_set_color (0);
    
    slrn_smg_refresh ();
@@ -418,7 +456,7 @@ static void draw_select_list (void)
    column = (SLtt_Screen_Cols - num_columns) / 2;
    
    slrn_set_color (FRAME_COLOR);
-   SLsmg_draw_box (row, column, num_rows + 2, num_columns + 2);
+   draw_box (row, column, num_rows + 2, num_columns + 2);
    SLsmg_gotorc (row + num_rows + 1, column + num_columns - 10);
    if (Is_Popup_Window)
      {
@@ -791,7 +829,7 @@ static int init_select_list_mode (int select_list)
 	adapt_shortcuts ();
      }
    
-   if (SLang_Error)
+   if (SLang_get_error ())
      return -1;
    
    Select_List_Mode_Cap.keymap = *kmap;
@@ -871,7 +909,7 @@ int slrn_select_list_mode (char *title,
    free_select_list (root, 0);
    slrn_pop_mode ();
    
-   slrn_message ("");
+   slrn_message ("%s", "");
 
    if (Select_List_Quit == -1)
      return -1;
@@ -922,7 +960,7 @@ int slrn_popup_win_mode (char *title, char *text)
 	  }
 	last = curr;
 	
-	if (NULL != (newline = strchr (text, '\n')))
+	if (NULL != (newline = slrn_strbyte (text, '\n')))
 	  {
 	     *newline++ = 0;
 	     if (*newline == 0)
@@ -932,7 +970,7 @@ int slrn_popup_win_mode (char *title, char *text)
 	
 	/* Here, we handle TABs (expand to spaces): */
 	tab = text;
-	while (NULL != (tab = strchr (tab, '\t')))
+	while (NULL != (tab = slrn_strbyte (tab, '\t')))
 	  {
 	     tab++;
 	     len += 8;
@@ -992,7 +1030,7 @@ int slrn_popup_win_mode (char *title, char *text)
    free_select_list (root, 1);
    slrn_pop_mode ();
    
-   slrn_message ("");
+   slrn_message ("%s", "");
    
    if (Select_List_Quit == -1)
      return -1;
