@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -32,14 +32,14 @@ extern void slrn_make_home_dirname (char *, char *, size_t);
 extern void slrn_redraw (void);
 extern void slrn_update_screen (void);
 extern int Slrn_Full_Screen_Update;
-extern int slrn_get_yesno (int, char *, ...);
-extern int slrn_get_yesno_cancel (char *str, ...);
+extern int slrn_get_yesno (int, char *, ...) ATTRIBUTE_PRINTF(2,3);
+extern int slrn_get_yesno_cancel (char *str, ...) ATTRIBUTE_PRINTF(1,2);
 extern void slrn_clear_message (void);
 extern char *slrn_print_percent (char *, SLscroll_Window_Type *, int);
 
 typedef char *(PRINTF_CB)(char, void *, int *, int *);
 extern void slrn_custom_printf (char *, PRINTF_CB, void *, int, int);
-extern void slrn_write_nchars (char *, unsigned int);
+extern void slrn_write_nbytes (char *, unsigned int);
 
 extern int slrn_set_display_format (char **, unsigned int, char *);
 extern unsigned int slrn_toggle_format (char **, unsigned int);
@@ -47,7 +47,7 @@ extern unsigned int slrn_toggle_format (char **, unsigned int);
 extern FILE *slrn_open_home_file (char *, char *, char *, size_t, int);
 extern VFILE *slrn_open_home_vfile (char *, char *, size_t);
 extern void slrn_suspend_cmd (void);
-extern int slrn_read_integer (char *, int *, int *);
+extern int slrn_read_artnum_int (char *, NNTP_Artnum_Type *, NNTP_Artnum_Type *);
 extern int slrn_read_input (char *, char *, char *, int, int);
 extern int slrn_read_input_no_echo (char *, char *, char *, int, int);
 extern int slrn_read_filename (char *, char *, char *, int, int);
@@ -56,13 +56,14 @@ extern void slrn_evaluate_cmd (void);
 extern void slrn_update_top_status_line (void);
 extern void slrn_set_color (int);
 extern char slrn_map_translated_char (char *, char *, char);
-extern char slrn_get_response (char *, char *, char *str, ...);
+extern char slrn_get_response (char *, char *, char *str, ...) ATTRIBUTE_PRINTF(3,4);
 extern int slrn_is_fqdn (char *);
 extern int slrn_init_readline (void);
 extern int slrn_check_batch (void);
 
 extern unsigned char *slrn_regexp_match (SLRegexp_Type *, char *);
 extern SLRegexp_Type *slrn_compile_regexp_pattern (char *);
+extern SLRegexp_Type *slrn_free_regexp (SLRegexp_Type *);
 
 #define MAX_HOST_NAME_LEN 256
 typedef struct
@@ -98,20 +99,30 @@ void slrn_set_input_chars (char *);
 
 extern int Slrn_Message_Present;
 
-#define SLRN_CONFIRM_CATCHUP	0x1
-#define SLRN_CONFIRM_PRINT	0x2
-#define SLRN_CONFIRM_POST	0x4
-#define SLRN_CONFIRM_URL	0x8
+#define SLRN_CONFIRM_CATCHUP	0x01
+#define SLRN_CONFIRM_PRINT	0x02
+#define SLRN_CONFIRM_POST	0x04
+#define SLRN_CONFIRM_URL	0x08
 #define SLRN_CONFIRM_QUIT	0x10
-#define SLRN_CONFIRM_ALL	0x1F
+#define SLRN_CONFIRM_ALL	0xFF
 extern int Slrn_User_Wants_Confirmation;
 extern void slrn_get_mouse_rc (int *, int *);
 #ifndef VMS
 extern char *Slrn_SendMail_Command;
 #endif
+
+#if SLANG_VERSION < 20000
+extern int SLang_get_error (void);
+extern int SLang_set_error (int);
+typedef struct SLKeyMap_List_Type SLkeymap_Type;
+typedef SLang_RLine_Info_Type SLrline_Type;
+extern SLkeymap_Type *SLrline_get_keymap (SLrline_Type *);
+typedef unsigned char SLuchar_Type;
+#endif
+
 extern SLKeyMap_List_Type *Slrn_RLine_Keymap;
 extern SLang_RLine_Info_Type *Slrn_Keymap_RLI;
-extern SLKeymap_Function_Type Slrn_Custom_Readline_Functions [];
+extern int slrn_rline_setkey (char *, char *, SLkeymap_Type *);
 
 extern void slrn_va_message (char *, va_list);
 
@@ -139,4 +150,5 @@ extern char *Slrn_Failed_Post_Filename;
 extern char *Slrn_Top_Status_Line;
 
 extern int Slrn_Abort_Unmodified;
+
 #endif				       /* _SLRN_MISC_H */
