@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -47,9 +47,11 @@
 #include "ttymsg.h"
 #include "snprintf.h"
 #include "server.h"
+#include "strutil.h"
+#include "common.h"
 
 void (*NNTP_Connection_Lost_Hook) (NNTP_Type *);
-int (*NNTP_Authorization_Hook) (char *, char **, char **);
+int (*NNTP_Authorization_Hook) (char *, int, char **, char **);
 int NNTP_Try_Authentication = 0;
 
 int Slrn_Broken_Xref;
@@ -59,6 +61,9 @@ static int _nntp_connect_server (NNTP_Type *);
 extern int Slrn_Force_Authentication;
 extern FILE *Slrn_Debug_Fp; /* we need this here when compiling slrnpull */
 
+/* Note: nntp_allocate_nntp returns a pointer to a static object.  Eventually,
+ * it should be changed to return a dynamic one.
+ */
 static NNTP_Type *nntp_allocate_nntp (void)
 {
    static NNTP_Type nn;
@@ -78,6 +83,12 @@ static void _nntp_deallocate_nntp (NNTP_Type *s)
 {
    if (s == NULL) return;
 
+   if (s->tcp != NULL)
+     sltcp_close (s->tcp);
+
+   /* Until nntp_allocate_nntp is modified to create a dynamic object, just
+    * zero the memory here.
+    */
    memset ((char *) s, 0, sizeof (NNTP_Type));
 }
 
@@ -209,6 +220,7 @@ static int _nntp_try_parse_timeout (char *str)
    /* Some servers now have NLS, so this might fail.
     * I'm trying to handle that case in nntp_server_cmd.
     */
+#if SLANG_VERSION < 20000
    static SLRegexp_Type re;
    unsigned char compiled_pattern_buf[256];
 
@@ -220,8 +232,19 @@ static int _nntp_try_parse_timeout (char *str)
    (void) SLang_regexp_compile (&re);
    if (NULL == SLang_regexp_match ((unsigned char *) str, strlen (str), &re))
      return -1;
-
    return 0;
+#else
+   int status = -1;
+   SLRegexp_Type *re = SLregexp_compile ("time.*out", SLREGEXP_CASELESS);
+   if (re != NULL)
+     {
+	if (NULL != SLregexp_match (re, str, strlen (str)))
+	  status = 0;
+	
+	SLregexp_free (re);
+     }
+   return status;
+#endif
 }
 
 int nntp_get_server_response (NNTP_Type *s)
@@ -326,7 +349,7 @@ int nntp_server_cmd (NNTP_Type *s, char *cmd)
 	     if ((code == ERR_NOAUTH) && NNTP_Try_Authentication)
 	       {
 		  NNTP_Try_Authentication = 0;
-		  if (-1 == nntp_authorization (s))
+		  if (-1 == nntp_authorization (s, 1))
 		    return -1;
 		  tried_auth = 1;
 		  continue;
@@ -358,7 +381,7 @@ int nntp_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_server_cmd (s, buf);
@@ -370,7 +393,7 @@ int nntp_start_server_vcmd (NNTP_Type *s, char *fmt, ...)
    va_list ap;
 
    va_start (ap, fmt);
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    va_end (ap);
 
    return nntp_start_server_cmd (s, buf);
@@ -395,23 +418,23 @@ int nntp_close_server (NNTP_Type *s)
    if (Slrn_Debug_Fp != NULL)
      fputs ("!Closing the server connection.\n", Slrn_Debug_Fp);
    
-   (void) sltcp_close (s->tcp);
-
-   s->tcp = NULL;
-   s->init_state = 0;
-
    _nntp_deallocate_nntp (s);
 
    return 0;
 }
 
-int nntp_authorization (NNTP_Type *s)
+int nntp_authorization (NNTP_Type *s, int auth_reqd)
 {
-   char *name, *pass;
+   char *name = NULL, *pass = NULL;
+   int status = 0;
 
-   if ((NNTP_Authorization_Hook == NULL) ||
-       (-1 == (*NNTP_Authorization_Hook) (s->name, &name, &pass)) ||
-       ((name == NULL) || (pass == NULL)))
+   if (NNTP_Authorization_Hook != NULL)
+     status = (*NNTP_Authorization_Hook) (s->name, auth_reqd, &name, &pass);
+
+   if ((auth_reqd == 0) && (status == 0))
+     return 0;			       /* not needed and info not present */
+   
+   if ((status == -1) || (name == NULL) || (pass == NULL))
      {
 	slrn_exit_error (_("Authorization needed, but could not determine username / password."));
 	return -1;
@@ -497,7 +520,7 @@ static int _nntp_connect_server (NNTP_Type *s)
    if (s->code == OK_NOPOST)
      s->can_post = 0;
    
-   if (Slrn_Force_Authentication && (-1 == nntp_authorization (s)))
+   if (-1 == nntp_authorization (s, Slrn_Force_Authentication))
      return -1;
 
    slrn_message_now (_("Connected to host.  %s"),
@@ -673,14 +696,28 @@ NNTP_Type *nntp_open_server (char *host, int port)
    return s;
 }
 
+/* This function removes the leading '.' from dot-escaped lines */
 int nntp_read_line (NNTP_Type *s, char *buf, unsigned int len)
 {
    if (-1 == nntp_gets_server (s, buf, len))
      return -1;
 
-   if ((buf[0] == '.') && (buf[1] == 0))
-     return 0;
-
+   if (buf[0] == '.')
+     {
+	char ch = buf[1];
+	if (ch == 0)
+	  return 0;
+	if (ch == '.')
+	  {
+	     char *b = buf;
+	     do
+	       {
+		  ch = b[1];
+		  *b++ = ch;
+	       }
+	     while (ch != 0);
+	  }
+     }
    return 1;
 }
 
@@ -744,7 +781,8 @@ static int _nntp_probe_server (NNTP_Type *s, char *cmd)
    return 1;
 }
 
-#define PROBE_XCMD(s, var,cmd) (((var) != -1) ? (var) : ((var) = _nntp_probe_server ((s),(cmd))))
+#define PROBE_XCMD(s, var, cmd) \
+   (((var) != -1) ? (var) : ((var) = _nntp_probe_server ((s),(cmd))))
 
 int nntp_has_cmd (NNTP_Type *s, char *cmd)
 {
@@ -758,23 +796,35 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
 
 	ret = nntp_server_cmd (s, cmd);
 
-	if ((ret == ERR_COMMAND) || (ret == ERR_ACCESS))
-	  s->can_xhdr = 0;
-	else if ((ret == OK_HEAD) &&
-		 (1 == nntp_read_line (s, buf, sizeof (buf))))
+	switch (ret)
 	  {
-	     char *p = buf;
-	     nntp_discard_output (s);
-	     while (*p && (*p != ' ')) p++;
-	     while (*p == ' ') p++;
-	     if (!strcmp (p, "(none)"))
-	       s->can_xhdr = 0;
-	     else
-	       s->can_xhdr = 1;
+	   default:
+	     s->can_xhdr = 0;
+	     break;
+
+	   case ERR_CMDSYN:
+	   case ERR_NCING:
+	     s->can_xhdr = 1;
+	     break;
+	     
+	   case ERR_COMMAND:
+	   case ERR_ACCESS:
+	     s->can_xhdr = 0;
+	     break;
+
+	   case OK_HEAD:
+	     s->can_xhdr = 1;
+	     if (1 == nntp_read_line (s, buf, sizeof (buf)))
+	       {
+		  char *p = buf;
+		  nntp_discard_output (s);
+		  while (*p && (*p != ' ')) p++;
+		  while (*p == ' ') p++;
+		  if (!strcmp (p, "(none)"))
+		    s->can_xhdr = 0;
+	       }
+	     break;
 	  }
-	else if (strlen (cmd) == 4)
-	  s->can_xhdr = 1;
-	
 	return s->can_xhdr;
      }
 
@@ -787,57 +837,56 @@ int nntp_has_cmd (NNTP_Type *s, char *cmd)
    return _nntp_probe_server (s, cmd);
 }
 
-static int _nntp_num_or_msgid_cmd (NNTP_Type *s, char *cmd, int n, char *msgid)
+static int _nntp_num_or_msgid_cmd (NNTP_Type *s, char *cmd, NNTP_Artnum_Type n, char *msgid)
 {
    if ((n != -1) && ((Slrn_Broken_Xref == 0) || (msgid == NULL)))
-     return nntp_server_vcmd (s, "%s %d", cmd, n);
+     return nntp_server_vcmd (s, ("%s " NNTP_FMT_ARTNUM), cmd, n);
+   else if (msgid == NULL)
+     return nntp_server_cmd(s, cmd);
    else
-     {
-	if (msgid == NULL) msgid = "";
-	return nntp_server_vcmd (s, "%s %s", cmd, msgid);
-     }
+     return nntp_server_vcmd (s, "%s %s", cmd, msgid);
 }
 
-int nntp_head_cmd (NNTP_Type *s, int n, char *msgid, int *real_id)
+int nntp_head_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid, NNTP_Artnum_Type *real_id)
 {
    int status;
 
    status = _nntp_num_or_msgid_cmd (s, "HEAD", n, msgid);
    if ((status == OK_HEAD) && (real_id != NULL))
-     *real_id = atoi(s->rspbuf + 4);
+     *real_id = NNTP_STR_TO_ARTNUM(s->rspbuf + 4);
    return status;
 }
 
-int nntp_xover_cmd (NNTP_Type *s, int min, int max)
+int nntp_xover_cmd (NNTP_Type *s, NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   return nntp_server_vcmd (s, "XOVER %d-%d", min, max);
+   return nntp_server_vcmd (s, "XOVER " NNTP_FMT_ARTRANGE, min, max);
 }
 
-int nntp_xhdr_cmd (NNTP_Type *s, char *field, int min, int max)
+int nntp_xhdr_cmd (NNTP_Type *s, char *field, NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   return nntp_server_vcmd (s, "XHDR %s %d-%d", field, min, max);
+   return nntp_server_vcmd (s, "XHDR %s " NNTP_FMT_ARTRANGE, field, min, max);
 }
 
-int nntp_article_cmd (NNTP_Type *s, int n, char *msgid)
+int nntp_article_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid)
 {
    return _nntp_num_or_msgid_cmd (s, "ARTICLE", n, msgid);
 }
 
-int nntp_next_cmd (NNTP_Type *s, int *n)
+int nntp_next_cmd (NNTP_Type *s, NNTP_Artnum_Type *n)
 {
    int status;
 
    if ((OK_NEXT == (status = nntp_server_cmd (s, "NEXT")))
        && (n != NULL))
-     *n = atoi (s->rspbuf + 4);
+     *n = NNTP_STR_TO_ARTNUM (s->rspbuf + 4);
 
    return status;
 }
 
-int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
+int nntp_select_group (NNTP_Type *s, char *name, NNTP_Artnum_Type *minp, NNTP_Artnum_Type *maxp)
 {
-   int estim;
-   int min, max;
+   NNTP_Artnum_Type estim;
+   NNTP_Artnum_Type min, max;
    
    switch (nntp_server_vcmd (s, "GROUP %s", name))
      {
@@ -846,7 +895,7 @@ int nntp_select_group (NNTP_Type *s, char *name, int *minp, int *maxp)
 
       case OK_GROUP:
 
-	if (3 != sscanf(s->rspbuf + 4, "%d %d %d", &estim, &min, &max))
+	if (3 != sscanf (s->rspbuf + 4, NNTP_FMT_ARTNUM_3, &estim, &min, &max))
 	  return -1;
 
 	if (minp != NULL) *minp = min;
@@ -904,7 +953,8 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
    i = prev;
    while (i < n)
      {
-	int estim;
+	NNTP_Artnum_Type estim;
+
 	prev = i;
 	switch (nntp_get_server_response (s))
 	  {
@@ -921,7 +971,7 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	     if (NNTP_Try_Authentication)
 	       {
 		  NNTP_Try_Authentication = 0;
-		  if (-1 == nntp_authorization (s))
+		  if (-1 == nntp_authorization (s, 1))
 		    return -1;
 		  max_tries++;
 		  goto start_over;
@@ -930,7 +980,7 @@ int nntp_refresh_groups (NNTP_Type *s, Slrn_Group_Range_Type *gr, int n)
 	       return -1;
 	     
 	   case OK_GROUP:
-	     if (3 != sscanf (s->rspbuf + 4, "%d %d %d", &estim,
+	     if (3 != sscanf (s->rspbuf + 4, NNTP_FMT_ARTNUM_3, &estim,
 			      &(gr[i].min), &(gr[i].max)))
 	       {
 		  s->flags |= reconnect;
@@ -975,16 +1025,17 @@ int nntp_end_post (NNTP_Type *s)
    return nntp_get_server_response (s);
 }
 
-int nntp_xpat_cmd (NNTP_Type *s, char *hdr, int rmin, int rmax, char *pat)
+int nntp_xpat_cmd (NNTP_Type *s, char *hdr, NNTP_Artnum_Type rmin, NNTP_Artnum_Type rmax, char *pat)
 {
    if (0 == PROBE_XCMD(s, s->can_xpat, "XPAT"))
      return ERR_COMMAND;
 
-   return nntp_server_vcmd (s, "XPAT %s %d-%d *%s*", hdr, rmin, rmax, pat);
+   return nntp_server_vcmd (s, ("XPAT %s " NNTP_FMT_ARTRANGE " *%s*"), 
+			    hdr, rmin, rmax, pat);
 }
 
 /* hdr should not include ':' */
-int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int buflen)
+int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, NNTP_Artnum_Type num, char *buf, unsigned int buflen)
 {
    char tmpbuf[1024];
    int found;
@@ -999,7 +1050,7 @@ int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int
      {
 	char *b, ch;
 
-	if (-1 == nntp_server_vcmd (s, "XHDR %s %d", hdr, num))
+	if (-1 == nntp_server_vcmd (s, ("XHDR %s " NNTP_FMT_ARTNUM), hdr, num))
 	  return -1;
 
 	if (OK_HEAD != s->code)
@@ -1045,7 +1096,7 @@ int nntp_one_xhdr_cmd (NNTP_Type *s, char *hdr, int num, char *buf, unsigned int
 	char *b;
 
 	if (found
-	    || slrn_case_strncmp ((unsigned char *) tmpbuf, (unsigned char *) hdr, colon)
+	    || slrn_case_strncmp ( tmpbuf,  hdr, colon)
 	    || (tmpbuf[colon] != ':'))
 	  continue;
 
@@ -1084,7 +1135,7 @@ int nntp_listgroup (NNTP_Type *s, char *group)
    /*  OK_GROUP desired */
 }
 
-int nntp_body_cmd (NNTP_Type *s, int n, char *msgid)
+int nntp_body_cmd (NNTP_Type *s, NNTP_Artnum_Type n, char *msgid)
 {
    return _nntp_num_or_msgid_cmd (s, "BODY", n, msgid);
 }
