@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -44,9 +44,12 @@
 # include "hash.h"
 #endif
 
+#include "strutil.h"
 #include "art.h"
 #include "xover.h"
 #include "score.h"
+#include "sortdate.h"
+#include "common.h"
 
 /* The score file will implement article scoring for slrn.  Basically the idea 
  * is that the user will provide a set of regular expressions that
@@ -116,8 +119,12 @@ PScore_Type;
 
 typedef struct
 {
+#if SLANG_VERSION < 20000
    SLRegexp_Type regexp;
    unsigned char buf[512];	       /* for compiled pattern */
+#else
+   SLRegexp_Type *regexp;
+#endif
 }
 Our_SLRegexp_Type;
 
@@ -132,7 +139,11 @@ typedef struct Score_Regexp_Type
      {
 	int ival;		       /* used by certain headers */
 	Our_SLRegexp_Type re;
+#if SLANG_VERSION < 20000
 	SLsearch_Type se;
+#else
+	SLsearch_Type *se;
+#endif
 	struct Score_Regexp_Type *srt;
      }
    search;
@@ -153,7 +164,11 @@ Score_Type;
 
 typedef struct Group_Score_Name_Type
 {
+#if SLANG_VERSION < 20000
    SLRegexp_Type group_regexp;
+#else
+   SLRegexp_Type *group_regexp;
+#endif
    char name[MAX_GROUP_NAME_LEN];      /* group name or pattern */
    unsigned char buf[MAX_GROUP_REGEXP_SIZE];/* for compiled pattern */
    struct Group_Score_Name_Type *next;
@@ -183,10 +198,10 @@ static Slrn_Score_Debug_Info_Type *append_score_debug_info
 {
    Slrn_Score_Debug_Info_Type *newentry;
    
-   if ((newentry = (Slrn_Score_Debug_Info_Type *)slrn_malloc
-	(sizeof (Slrn_Score_Debug_Info_Type), 0, 0)) == NULL)
+   if (NULL == (newentry = (Slrn_Score_Debug_Info_Type *)
+		slrn_malloc (sizeof (Slrn_Score_Debug_Info_Type), 0, 0)))
      return NULL;
-   
+
    newentry->filename = sc->filename;
    newentry->description = sc->description;
    newentry->linenumber = sc->linenumber;
@@ -327,7 +342,7 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
           }
 
         len = strlen (s);
-
+#if SLANG_VERSION < 20000
         if (srt->do_osearch)
           {
              SLsearch_Type *se = &srt->search.se;
@@ -350,12 +365,17 @@ static int match_srt (Slrn_Header_Type *h, Score_Regexp_Type *srt,
                }
           }
         else
+#endif
           {
              SLRegexp_Type *re;
-
+#if SLANG_VERSION < 20000
              re = &srt->search.re.regexp;
              if ((len < re->min_length)
                  || (NULL == SLang_regexp_match ((unsigned char *)s, len, re)))
+#else
+	     re = srt->search.re.regexp;
+	     if (NULL == SLregexp_match (re, s, len))
+#endif
                {
                   if (srt->not_flag == 0)
                     {
@@ -463,8 +483,6 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup,
 static int compile_psrt (PScore_Regexp_Type *psrt, Score_Regexp_Type *srt,
 			 int *generic)
 {
-   SLRegexp_Type *re;
-
    while (psrt != NULL)
      {
         unsigned int flags = psrt->flags;
@@ -497,31 +515,38 @@ static int compile_psrt (PScore_Regexp_Type *psrt, Score_Regexp_Type *srt,
                }
              else
                {
-                  re = &srt->search.re.regexp;
+#if SLANG_VERSION < 20000
+		  SLRegexp_Type *re = &srt->search.re.regexp;
                   re->pat = psrt->ireg.regexp_str;
                   re->buf = srt->search.re.buf;
                   re->buf_len = sizeof (srt->search.re.buf);
                   re->case_sensitive = 0;
-
-                  if (0 != SLang_regexp_compile(re))
+		  if (0 != SLang_regexp_compile(re))
                     {
                        return -1;
                     }
-
+#else
+		  if (NULL == (srt->search.re.regexp = SLregexp_compile ((char *) psrt->ireg.regexp_str, SLREGEXP_CASELESS)))
+		    return -1;
+#endif
+#if SLANG_VERSION < 20000
                   /* If an ordinary search is ok, use it. */
                   if (re->osearch)
                     {
                        srt->do_osearch = 1;
                        SLsearch_init ((char *) psrt->ireg.regexp_str, 1, 0, &srt->search.se);
                     }
+#endif
                }
           }
 		
         psrt = psrt->next;
         if (psrt == NULL) break;
 
-        srt->next = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type), 1, 0);
-        if (NULL == (srt = srt->next)) return -1;
+        if (NULL == (srt->next = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type), 1, 0)))
+	  return -1;
+
+        srt = srt->next;
      } /* while (psrt != NULL)*/
    return 0;
 }
@@ -558,14 +583,53 @@ static int chain_group_regexp (PScore_Type *pst, int *generic)
 
 char *Slrn_Scorefile_Open = NULL;
 
+static void free_srt (Score_Regexp_Type *);
+static void free_srt_internal (Score_Regexp_Type *srt)
+{
+   if (srt == NULL)
+     return;
+
+   switch (srt->header_type)
+     {
+      case SCORE_LINES:
+      case SCORE_BYTES:
+      case SCORE_HAS_BODY:
+      case SCORE_AGE:
+	/* Use integer or boolean comparisons */
+	break;
+
+      case SCORE_SUB_AND:
+      case SCORE_SUB_OR:
+	free_srt (srt->search.srt);
+	break;
+	     
+      case SCORE_SUBJECT:
+      case SCORE_FROM:
+      case SCORE_DATE:
+      case SCORE_MESSAGE_ID:
+      case SCORE_XREF:
+      case SCORE_REFERENCES:
+      case SCORE_NEWSGROUP:
+      case SCORE_GENERIC:
+#if SLANG_VERSION >= 20000
+	if (srt->do_osearch)
+	  {
+	     if (srt->search.se != NULL)
+	       SLsearch_delete (srt->search.se);
+	  }
+	else if (srt->search.re.regexp != NULL)
+	  SLregexp_free (srt->search.re.regexp);
+#endif
+	break;
+     }
+}
+
 static void free_srt (Score_Regexp_Type *srt)
 {
    while (srt != NULL)
      {
 	Score_Regexp_Type *srt_next = srt->next;
-	if ((srt->header_type == SCORE_SUB_AND) || 
-	    (srt->header_type == SCORE_SUB_OR))
-	  free_srt (srt->search.srt);
+	free_srt_internal (srt);
 	SLFREE (srt);
 	srt = srt_next;
      }
@@ -580,9 +644,8 @@ static void free_group_chain (void)
 	Score_Type *next = Score_Root->next;
 	srt = &Score_Root->regexp_list;
 	/* first not malloced; free subscores only: */
-	if ((srt->header_type == SCORE_SUB_AND) ||
-	    (srt->header_type == SCORE_SUB_OR))
-	  free_srt (srt->search.srt);
+	free_srt_internal (srt);
+
 	free_srt (srt->next);
 	SLFREE (Score_Root);
 	Score_Root = next;
@@ -625,10 +688,14 @@ int slrn_open_score (char *group_name)
 	match = 0;
 	while ((gsnt != NULL) && (match == 0))
 	  {
+#if SLANG_VERSION < 20000
 	     SLRegexp_Type *re = &gsnt->group_regexp;
 
 	     match = ((re->min_length <= n)
 		      && (NULL != SLang_regexp_match ((unsigned char *) group_name, n, re)));
+#else
+	     match = (NULL != SLregexp_match (gsnt->group_regexp, group_name, n));
+#endif
 	     gsnt = gsnt->next;
 	  }
 
@@ -668,8 +735,8 @@ void slrn_close_score (void)
 
 
 
-static int add_group_regexp (PScore_Regexp_Type *psrt, unsigned char *str,
-			     unsigned char *keyword, unsigned int type,
+static int add_group_regexp (PScore_Regexp_Type *psrt, char *str,
+			     char *keyword, unsigned int type,
 			     int not_flag)
 {
    unsigned int len;
@@ -683,23 +750,23 @@ static int add_group_regexp (PScore_Regexp_Type *psrt, unsigned char *str,
    psrt->header_type = type;
    if ((type != SCORE_SUB_AND) && (type != SCORE_SUB_OR))
      {
-	len = (unsigned int) (slrn_trim_string ((char *) str) - (char *) str);
+	len = (unsigned int) (slrn_trim_string (str) - str);
 	if (0 == len) return -1;
 	
 	if ((type == SCORE_LINES) || (type == SCORE_BYTES) || (type == SCORE_HAS_BODY))
 	  {
-	     psrt->ireg.ival = atoi((char *)str);
+	     psrt->ireg.ival = atoi(str);
 	     psrt->flags |= USE_INTEGER;
 	  }
 	else if (type == SCORE_AGE)
 	  {
-	     psrt->ireg.ival = time(NULL) - atoi((char *)str) * 86400;
+	     psrt->ireg.ival = time(NULL) - atoi(str) * 86400;
 	     psrt->flags |= USE_INTEGER;
 	  }
 	else
-	  psrt->ireg.regexp_str = (unsigned char *) slrn_safe_strmalloc ((char *)str);
+	  psrt->ireg.regexp_str = (unsigned char *) slrn_safe_strmalloc (str);
 	
-	strncpy (psrt->keyword, (char *) keyword, MAX_KEYWORD_LEN);
+	strncpy (psrt->keyword, keyword, MAX_KEYWORD_LEN);
 	psrt->keyword[MAX_KEYWORD_LEN - 1] = 0;
      }
    else
@@ -725,11 +792,12 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
    comma = group;
    while (comma != NULL)
      {
+#if SLANG_VERSION < 20000
 	SLRegexp_Type *re;
-	
+#endif
 	group = slrn_skip_whitespace (group);
 	
-	comma = slrn_strchr ((char *) group, ',');
+	comma = slrn_strbyte ((char *) group, ',');
 	if (comma != NULL) 
 	  {
 	     *comma++ = 0;
@@ -754,6 +822,7 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
 	strncpy (gsnt->name, group, MAX_GROUP_NAME_LEN - 1);
 	/* Note: because of the memset, this string is null terminated. */
    
+#if SLANG_VERSION < 20000
 	re = &gsnt->group_regexp;
 	re->pat = (unsigned char *) slrn_fix_regexp ((char *)group);
 	re->buf = gsnt->buf;
@@ -763,7 +832,10 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
 	  {
 	     return -1;
 	  }
-	
+#else
+	if (NULL == (gsnt->group_regexp = SLregexp_compile (slrn_fix_regexp ((char *)group), SLREGEXP_CASELESS)))
+	  return -1;
+#endif
 	if (comma != NULL)
 	  {
 	     Group_Score_Name_Type *gsnt1;
@@ -870,6 +942,10 @@ static void free_group_scores (void)
 	while (gsnt != NULL)
 	  {
 	     Group_Score_Name_Type *next = gsnt->next;
+#if SLANG_VERSION >= 20000
+	     if (gsnt->group_regexp != NULL) 
+	       SLregexp_free (gsnt->group_regexp);
+#endif
 	     SLFREE (gsnt);
 	     gsnt = next;
 	  }
@@ -890,16 +966,16 @@ static void free_group_scores (void)
      }
 }
 
-static int has_score_expired (unsigned char *s, unsigned long today)
+static int has_score_expired (char *s, unsigned long today)
 {
    unsigned long mm, dd, yyyy;
    unsigned long score_time;
    
-   s = (unsigned char *) slrn_skip_whitespace ((char *) s);
+   s = slrn_skip_whitespace (s);
    if (*s == 0) return 0;
    
-   if (((3 != sscanf ((char *) s, "%lu/%lu/%lu", &mm, &dd, &yyyy))
-	&& (3 != sscanf ((char *) s, "%lu-%lu-%lu", &dd, &mm, &yyyy)))
+   if (((3 != sscanf (s, "%lu/%lu/%lu", &mm, &dd, &yyyy))
+	&& (3 != sscanf (s, "%lu-%lu-%lu", &dd, &mm, &yyyy)))
        || (dd > 31)
        || (mm > 12)
        || (yyyy < 1900))
@@ -936,7 +1012,11 @@ typedef struct
    int score_has_expired;
    unsigned int pscore_flags;
    PScore_Type *pst;
+#if SLANG_VERSION < 20000
    SLPreprocess_Type pt;
+#else
+   SLprep_Type *pt;
+#endif
 }
 Score_Context_Type;
 
@@ -997,25 +1077,29 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
     
     while (fgets (line, sizeof (line) - 1, fp))
      {
-	unsigned char *lp;
+	char *lp;
 	int ret, not_flag;
 	
 	(*linenum)++;
 
 	psrt = NULL;
 	
+#if SLANG_VERSION < 20000
 	if (0 == SLprep_line_ok (line, &c->pt))
+#else
+	if (0 == SLprep_line_ok (line, c->pt))
+#endif
 	  continue;
 
-	lp = (unsigned char *) slrn_skip_whitespace (line);
+	lp = slrn_skip_whitespace (line);
 	if ((*lp == '#') || (*lp == '%') || (*lp <= ' ')) continue;
 
 	if (sub_psrt == NULL)
 	  {
-	     if ((0 == slrn_case_strncmp (lp, (unsigned char *) "include", 7))
+	     if ((0 == slrn_case_strncmp (lp,  "include", 7))
 		 && ((lp[7] == ' ') || (lp[7] == '\t')))
 	       {
-		  if (0 == handle_include_line (file, (char *)lp + 7, c))
+		  if (0 == handle_include_line (file, lp + 7, c))
 		    continue;
 		  
 		  score_error (_("Error handling INCLUDE line"), line, *linenum,
@@ -1031,13 +1115,13 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		  gmax = g + sizeof (c->group);
 		  
 		  lp++;
-		  lp = (unsigned char *) slrn_skip_whitespace ((char *)lp);
+		  lp = slrn_skip_whitespace (lp);
 		  
 		  c->gnt_not_flag = 0;
 		  if (*lp == '~')
 		    {
 		       c->gnt_not_flag = 1;
-		       lp = (unsigned char *) slrn_skip_whitespace ((char *)lp + 1);
+		       lp = slrn_skip_whitespace (lp + 1);
 		    }
 		  
 		  while (((ch = *lp++) != 0)
@@ -1057,22 +1141,22 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		  continue;
 	       }
         
-	     if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
+	     if (!slrn_case_strncmp (lp, "Score:", 6))
 	       {
-		  unsigned char *lpp = lp + 6, *eol;
+		  char *lpp = lp + 6, *eol;
 		  c->pscore_flags = 0;
 		  if (*lpp == ':')
 		    {
 		       lpp++;
 		       c->pscore_flags |= SCORE_IS_OR_TYPE;
 		    }
-		  lpp = (unsigned char *) slrn_skip_whitespace ((char *) lpp);
+		  lpp = slrn_skip_whitespace (lpp);
 		  if (*lpp == '=')
 		    {
 		       c->pscore_flags |= RETURN_THIS_SCORE;
 		       lpp++;
 		    }
-		  c->score = atoi ((char *)lpp);
+		  c->score = atoi (lpp);
 		  c->start_new_score = 1;
 
                   if (description != NULL)
@@ -1080,17 +1164,17 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 		       slrn_free (description);
 		       description = NULL;
 		    }
-                  eol = (unsigned char *) slrn_strchr ((char *)lpp, '#');
+                  eol = slrn_strbyte (lpp, '#');
 		  if (eol == NULL)
-		    lpp = (unsigned char *) slrn_strchr ((char *)lpp, '%');
+		    lpp = slrn_strbyte (lpp, '%');
 		  else lpp = eol;
                   if (lpp != NULL)
 		    {
-		       lpp = (unsigned char *) slrn_skip_whitespace ((char *)lpp + 1);
-		       eol = lpp + strlen ((char*)lpp) - 1;
-		       while ((eol >= lpp) && (*eol <= 32))
+		       lpp = slrn_skip_whitespace (lpp + 1);
+		       eol = lpp + strlen (lpp) - 1;
+		       while ((eol >= lpp) && ((unsigned char)*eol <= 32))
 			 *eol-- = '\0';
-		       description = slrn_safe_strmalloc ((char *)lpp);
+		       description = slrn_safe_strmalloc (lpp);
 		    }
 
 		  continue;
@@ -1098,7 +1182,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	     
 	     if (c->start_new_score)
 	       {
-		  if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
+		  if (!slrn_case_strncmp (lp,  "Expires:", 8))
 		    {
 		       ret = has_score_expired (lp + 8, c->today);
 		       if (ret == -1)
@@ -1164,29 +1248,29 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	ret = -1;
 	psrt = (PScore_Regexp_Type *) slrn_safe_malloc (sizeof (PScore_Regexp_Type));
 	/* Otherwise the line is a kill one */
-	if (!slrn_case_strncmp (lp, (unsigned char *)"Subject:", 8))
+	if (!slrn_case_strncmp (lp, "Subject:", 8))
 	  ret = add_group_regexp (psrt, lp + 7, lp, SCORE_SUBJECT, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"From:", 5))
+	else if (!slrn_case_strncmp (lp, "From:", 5))
 	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_FROM, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Xref:", 5))
+	else if (!slrn_case_strncmp (lp, "Xref:", 5))
 	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_XREF, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Newsgroup:", 10))
+	else if (!slrn_case_strncmp (lp, "Newsgroup:", 10))
 	  ret = add_group_regexp (psrt, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"References:", 11))
+	else if (!slrn_case_strncmp (lp, "References:", 11))
 	  ret = add_group_regexp (psrt, lp + 10, lp, SCORE_REFERENCES, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Lines:", 6))
+	else if (!slrn_case_strncmp (lp, "Lines:", 6))
 	  ret = add_group_regexp (psrt, lp + 5, lp, SCORE_LINES, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Date:", 5))
+	else if (!slrn_case_strncmp (lp, "Date:", 5))
 	  ret = add_group_regexp (psrt, lp + 4, lp, SCORE_DATE, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Age:", 4))
+	else if (!slrn_case_strncmp (lp, "Age:", 4))
 	  ret = add_group_regexp (psrt, lp + 3, lp, SCORE_AGE, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Bytes:", 6))
+	else if (!slrn_case_strncmp (lp, "Bytes:", 6))
 	  ret = add_group_regexp (psrt, lp + 5, lp, SCORE_BYTES, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Has-Body:", 9))
+	else if (!slrn_case_strncmp (lp, "Has-Body:", 9))
 	  ret = add_group_regexp (psrt, lp + 8, lp, SCORE_HAS_BODY, not_flag);
-	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
+	else if (!slrn_case_strncmp (lp, "Message-Id:", 11))
 	  ret = add_group_regexp (psrt, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
-        else if (!slrn_case_strncmp (lp, (unsigned char *)"{:", 2))
+        else if (!slrn_case_strncmp (lp, "{:", 2))
 	  {
             if (lp[2] ==':')
 	      {
@@ -1198,7 +1282,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	      }
 	    if (phrase_score_file(file, fp, c, linenum, psrt))  return -1;
           }
-        else if (!slrn_case_strncmp (lp, (unsigned char *)"}", 1))
+        else if (!slrn_case_strncmp (lp, "}", 1))
 	  {
 	     SLFREE (psrt);
 	     if (sub_psrt != NULL)
@@ -1211,7 +1295,7 @@ static int phrase_score_file (char *file, FILE *fp, Score_Context_Type *c,
 	  }
 	else
 	  {
-	     unsigned char *lpp = lp;
+	     char *lpp = lp;
 	     while (*lpp && (*lpp != ':')) lpp++;
 	     if (*lpp != ':')
 	       {
@@ -1328,9 +1412,18 @@ int slrn_read_score_file (char *file)
    sc.group[0] = '*';
    sc.group[1] = 0;
 
-   (void) SLprep_open_prep (&sc.pt);
+#if SLANG_VERSION < 20000
+   if (-1 == SLprep_open_prep (&sc.pt))
+#else
+   if (NULL == (sc.pt = SLprep_new ()))
+#endif
+     return -1;
    status = read_score_file_internal (file, &sc);
+#if SLANG_VERSION < 20000
    SLprep_close_prep (&sc.pt);
+#else
+   SLprep_delete (sc.pt);
+#endif
    
    if (status == -1)
      Slrn_Apply_Score = 0;
