@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -73,6 +73,7 @@
 
 #include <errno.h>
 
+#include "common.h"
 #include "util.h"
 #include "server.h"
 #include "slrn.h"
@@ -82,11 +83,12 @@
 #include "art.h"
 #include "score.h"
 #include "snprintf.h"
-#include "chmap.h"
+#include "charset.h"
 #include "post.h"
 #include "xover.h"
 #include "mime.h"
 #include "hooks.h"
+#include "strutil.h"
 
 #if SLRN_USE_SLTCP
 # include "sltcp.h"
@@ -95,9 +97,7 @@
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
-#if SLRN_HAS_SLANG
-# include "interp.h"
-#endif
+#include "interp.h"
 #ifdef __os2__
 # define INCL_VIO
 # include <os2.h>
@@ -111,6 +111,10 @@
 
 /*{{{ Global Variables */
 
+#if SLANG_VERSION >= 20000
+int Slrn_UTF8_Mode = 0;
+#endif
+
 int Slrn_TT_Initialized = 0;
 
 /* If -1, force mouse.  If 1 the mouse will be used on in XTerm.  If 0, 
@@ -136,12 +140,12 @@ int Slrn_Default_Post_Obj = SLRN_DEFAULT_POST_OBJ;
 FILE *Slrn_Debug_Fp = NULL;
 
 /* You need to call slrn_init_graphic_chars before using these */
-SLsmg_Char_Type Graphic_LTee_Char;
-SLsmg_Char_Type Graphic_UTee_Char;
-SLsmg_Char_Type Graphic_LLCorn_Char;
-SLsmg_Char_Type Graphic_HLine_Char;
-SLsmg_Char_Type Graphic_VLine_Char;
-SLsmg_Char_Type Graphic_ULCorn_Char;
+SLwchar_Type Graphic_LTee_Char;
+SLwchar_Type Graphic_UTee_Char;
+SLwchar_Type Graphic_LLCorn_Char;
+SLwchar_Type Graphic_HLine_Char;
+SLwchar_Type Graphic_VLine_Char;
+SLwchar_Type Graphic_ULCorn_Char;
 
 int Graphic_Chars_Mode = ALT_CHAR_SET_MODE;
 
@@ -153,6 +157,7 @@ static volatile int Want_Suspension;
 static volatile int Want_Window_Size_Change;
 static void perform_suspend (int);
 static int Current_Mouse_Mode;
+static int Ran_Startup_Hook;
 
 /*}}}*/
 /*{{{ Static Function Declarations */
@@ -854,9 +859,9 @@ static void perform_cleanup (void)
    slrn_close_grouplens ();
 #endif
 
-#if SLRN_HAS_SLANG
+   if (Ran_Startup_Hook)
+     (void) slrn_run_hooks (HOOK_QUIT, 0);
    (void) slrn_reset_slang ();
-#endif
 
 #if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
    (void) sltcp_close_sltcp ();
@@ -910,7 +915,19 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
-static void usage (char *extra) /*{{{*/
+void slrn_error (char *fmt, ...) /*{{{*/
+{
+   va_list ap;
+
+   va_start(ap, fmt);
+   slrn_verror (fmt, ap);
+   va_end (ap);
+}
+
+/*}}}*/
+
+
+static void usage (char *extra, int exit_status) /*{{{*/
 {
    printf (_("\
 Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
@@ -933,6 +950,7 @@ Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
 --debug FILE    Write debugging information to FILE\n\
 --help          Print this usage.\n\
 --kill-log FILE Keep a log of all killed articles in FILE\n\
+--show-config   Print configuration\n\
 --version       Show version and supported features\n\
 \n\
 NNTP mode has additional options; use \"slrn --nntp --help\" to display them.\n\
@@ -942,7 +960,7 @@ NNTP mode has additional options; use \"slrn --nntp --help\" to display them.\n\
      {
 	printf ("\n%s\n", extra);
      }
-   exit (1);
+   exit (exit_status);
 }
 
 /*}}}*/
@@ -968,7 +986,7 @@ static int parse_object_args (char *obj, char **argv, int argc) /*{{{*/
 	  slrn_exit_error (_("%s is not supported."), obj);
      }
    if ((num_parsed == 0) && (zero_ok == 0))
-     usage (NULL);
+     usage (NULL, 1);
    
    return num_parsed;
 }
@@ -1014,13 +1032,14 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    char file [SLRN_MAX_PATH_LEN];
    char *init_file = NULL;
    char *kill_logfile = NULL;
+   int print_config = 0;
 
 #if defined(HAVE_SETLOCALE) && defined(LC_ALL)
    (void) setlocale(LC_ALL, "");
 #endif
 #ifdef ENABLE_NLS
-   bindtextdomain(PACKAGE,LOCALEDIR);
-   textdomain(PACKAGE);
+   bindtextdomain(NLS_PACKAGE_NAME, NLS_LOCALEDIR);
+   textdomain (NLS_PACKAGE_NAME);
 #endif
 
    check_for_suspension ();
@@ -1028,27 +1047,6 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    (void) umask (077);
 #endif
    
-   /* These are already available if the interpreter is available */
-#if !SLRN_HAS_SLANG
-# ifdef __os2__
-   SLdefine_for_ifdef ("OS2");
-# else
-#  ifdef VMS
-   SLdefine_for_ifdef ("VMS");
-#  else
-#   ifdef __NT__
-   SLdefine_for_ifdef ("NT");
-#   else
-#    ifdef __WIN32__
-   SLdefine_for_ifdef ("WIN32");
-#    else
-   SLdefine_for_ifdef ("UNIX");
-#    endif
-#   endif
-#  endif
-# endif
-#endif
-
 #if 0
    if (NULL != getenv ("AUTOSUBSCRIBE"))
      Slrn_Unsubscribe_New_Groups = 0;
@@ -1074,7 +1072,12 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 #endif
 	     else if (!strcmp ("create", argv_i)) create_flag = 1;
 	     else if (!strcmp ("version", argv_i))
-	       slrn_show_version ();
+	       {
+		  slrn_show_version (stdout);
+		  exit (0);
+	       }
+	     else if (!strcmp ("show-config", argv_i))
+	       print_config = 1;
 	     else if ((!strcmp ("kill-log", argv_i)) &&
 		      (i + 1 < (unsigned int) argc))
 	       kill_logfile = argv[++i];
@@ -1088,7 +1091,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
                     slrn_exit_error (_("Unable to open %s for debugging."), argv[i]);
                   setbuf (Slrn_Debug_Fp, (char *) NULL);
                }
-	     else usage (NULL);
+	     else if (!strcmp ("help", argv_i))
+	       usage (NULL, 0);
+	     else usage (NULL, 1);
 	  }
 	else if (!strcmp ("-create", argv_i)) create_flag = 1;
 	else if (!strcmp ("-C", argv_i)) use_color = 1;
@@ -1129,30 +1134,30 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	  }
      }
    
-   fprintf (stdout, "slrn %s [%s]\n", Slrn_Version, Slrn_Date);
+   fprintf (stdout, "slrn %s\n", Slrn_Version_String);
 
    if (dsc_flag && create_flag)
      {
-	usage (_("The -d and --create flags must not be specified together."));
+	usage (_("The -d and --create flags must not be specified together."), 1);
      }
    
    if (Slrn_Batch == 0)
      {
+	Slrn_UTF8_Mode = SLutf8_enable (-1);
 	SLtt_get_terminfo ();
 	if (use_color == 1) SLtt_Use_Ansi_Colors = 1;
 	else if (use_color == -1) SLtt_Use_Ansi_Colors = 0;
      }
 
 #if SLRN_USE_SLTCP && SLRN_HAS_NNTP_SUPPORT
+   SLtcp_Verror_Hook = slrn_verror;
    /* This is necessary to ensure that gethostname works on Win32 */
    if (-1 == sltcp_open_sltcp ())
      slrn_exit_error (_("Error initializing SLtcp interface"));
 #endif
 
-#if SLRN_HAS_SLANG
    if (-1 == slrn_init_slang ())
-     fprintf (stderr, _("Error initializing S-Lang interpreter.\n"));
-#endif
+     slrn_exit_error (_("Error initializing S-Lang interpreter.\n"));
 
    slrn_startup_initialize ();
 
@@ -1172,9 +1177,9 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    slrn_init_hangup_signals (1);
 
 #ifdef VMS
-   slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s%s", SLRN_CONF_DIR, "slrn.rc");
 #else
-   slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "slrn.rc");
+   slrn_snprintf (file, sizeof (file), "%s/%s", SLRN_CONF_DIR, "slrn.rc");
 #endif
 
    /* Make sure terminal is initialized before setting colors.  The 
@@ -1208,38 +1213,15 @@ Could not read specified config file %s\n"), init_file);
    if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
    if (no_new_groups) Slrn_Check_New_Groups = 0;
 
-#if SLRN_HAS_CHARACTER_MAP
-   if (-1 == slrn_set_charset (Slrn_Charset))
-     {
-	slrn_chmap_show_supported ();
-	slrn_exit_error (_("Failed to select character set."));
-     }
-#endif
+   slrn_prepare_charset();
 
-#if SLRN_HAS_MIME
-   if (NULL == Slrn_Mime_Display_Charset)
+   if (print_config)
      {
-# if SLRN_HAS_CHARACTER_MAP
-	if (NULL != Slrn_Charset)
-	  {
-	     if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					(unsigned char *)"ibm852"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-2");
-	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					     (unsigned char *)"ibm857"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-9");
-	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					     (unsigned char *)"koi8"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("koi8-r");
-	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-					     (unsigned char *)"ibm737"))
-	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-7");
-	  }
-	if (NULL == Slrn_Mime_Display_Charset)
-# endif
-	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-1");
+	(void) putc ('\n', stdout);
+	slrn_show_version (stdout);
+	slrn_print_config (stdout);
+	slrn_quit (0);
      }
-#endif
 
 #ifdef SIGINT
    if (Slrn_TT_Initialized == 0)
@@ -1267,12 +1249,13 @@ Could not read specified config file %s\n"), init_file);
      {
 	slrn_exit_error (_("\
 Unable to find a valid hostname for constructing your e-mail address.\n\
-You will have to specify a hostname in your %s file.\n"), 
+You probably want to specify a hostname in your %s file.\n\
+Please see the \"slrn reference manual\" for full details.\n"), 
 			 SLRN_USER_SLRNRC_FILENAME);
      }
    if ((NULL == Slrn_User_Info.username)
        || (0 == *Slrn_User_Info.username)
-       || (NULL != slrn_strchr (Slrn_User_Info.username, '@')))
+       || (NULL != slrn_strbyte (Slrn_User_Info.username, '@')))
      {
 	slrn_exit_error (_("\
 Unable to find your user name.  This means that a valid 'From' header line\n\
@@ -1296,9 +1279,9 @@ cannot be constructed.  Try setting the USER environment variable.\n"));
      {
 	hlp_file = file;
 #ifdef VMS
-	slrn_snprintf (file, sizeof (file), "%s%s", SYSCONFDIR, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s%s", SLRN_CONF_DIR, "help.txt");
 #else
-	slrn_snprintf (file, sizeof (file), "%s/%s", SYSCONFDIR, "help.txt");
+	slrn_snprintf (file, sizeof (file), "%s/%s", SLRN_CONF_DIR, "help.txt");
 #endif
      }
 	
@@ -1370,10 +1353,8 @@ If you want to create %s, add command line options:\n\
    
    lock_file (1);
 
-#if SLRN_HAS_SLANG
    (void) slrn_run_hooks (HOOK_STARTUP, 0);
-#endif
-   
+   Ran_Startup_Hook = 1;
    
 #if SLRN_HAS_GROUPLENS
    if (Slrn_Server_Id != SLRN_SERVER_ID_NNTP)
@@ -1504,10 +1485,8 @@ static void run_winch_functions (int old_r, int old_c)
 	  (*mode->sigwinch_fun) (old_r, old_c);
      }
 
-#if SLRN_HAS_SLANG
-   if (SLang_Error == 0)
+   if (SLang_get_error() == 0)
      slrn_run_hooks (HOOK_RESIZE_SCREEN, 0);
-#endif
 }
 
 
@@ -1586,7 +1565,7 @@ int slrn_getkey (void)
 	
 	slrn_push_suspension (0);
 	r = SLsmg_get_row (); c = SLsmg_get_column ();
-	slrn_message (buf);
+	slrn_message ("%s", buf);
 	SLsmg_gotorc (r, c);
 	slrn_smg_refresh ();
 	slrn_pop_suspension ();
@@ -1594,6 +1573,9 @@ int slrn_getkey (void)
      }
    
    ch = SLang_getkey ();
+   if (ch == SLANG_GETKEY_ERROR)
+     slrn_exit_error (_("SLang_getkey failed"));
+	
    if (buf_len + 4 < sizeof (buf))
      {
 	if (ch == 0) 
@@ -1628,14 +1610,13 @@ void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
    key = SLang_do_key (map, slrn_getkey);
    Suspend_Sigtstp_Suspension = 0;
 
-   if (Slrn_Message_Present || SLang_Error) 
+   if (Slrn_Message_Present || SLang_get_error()) 
      {
-#if SLRN_HAS_SLANG
-	if (SLang_Error) SLang_restart (0);
-#endif
+	if (SLang_get_error()) SLang_restart (0);
 	slrn_clear_message ();
      }
-   SLang_Error = SLKeyBoard_Quit = 0;
+   SLKeyBoard_Quit = 0;
+   SLang_set_error (0);
    
    if ((key == NULL) || (key->type == 0))
      {
@@ -1660,8 +1641,6 @@ void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
      }
 
    /* Otherwise we have interpreted key. */
-#if SLRN_HAS_SLANG
-   
    last_key = key;
    last_map = map;
 
@@ -1670,7 +1649,6 @@ void slrn_do_keymap_key (SLKeyMap_List_Type *map) /*{{{*/
    if ((*key->f.s == '.')
        || !SLang_execute_function (key->f.s))
      SLang_load_string(key->f.s);
-#endif
 }
 
 /*}}}*/
@@ -1732,18 +1710,22 @@ int main (int argc, char **argv) /*{{{*/
 {
    if (-1 == main_init_and_parse_args (argc, argv))
      return 1;
-   
+ 
    if (-1 == slrn_select_group_mode ())
      return 1;
 
    slrn_push_suspension (1);
 
-#if SLRN_HAS_SLANG
    (void) slrn_run_hooks (HOOK_GROUP_MODE_STARTUP, 0);
-#endif
 
    if (Slrn_Batch) return 1;
-   
+
+   if (SLang_get_error ())
+     {
+	SLang_restart (1);	       /* prints any queued messages */
+	slrn_exit_error (_("An error was encountered during initialization."));
+     }
+
    while (Slrn_Current_Mode != NULL)
      {
 	if (SLKeyBoard_Quit)
@@ -1754,7 +1736,7 @@ int main (int argc, char **argv) /*{{{*/
 	
 	(void) slrn_handle_interrupts ();
 
-	if (SLang_Error || !SLang_input_pending(0))
+	if (SLang_get_error() || !SLang_input_pending(0))
 	  {
 	     slrn_update_screen ();
 	  }
