@@ -1,8 +1,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -27,6 +27,11 @@ extern int Slrn_TT_Initialized;
 extern int Slrn_Use_Flow_Control;
 extern int Slrn_Simulate_Graphic_Chars;
 extern int Slrn_Saw_Warning;
+#if SLANG_VERSION >= 20000
+# ifndef SLRL_DISPLAY_BUFFER_SIZE
+#  define SLRL_DISPLAY_BUFFER_SIZE 256
+# endif
+#endif
 
 extern int slrn_getkey (void);
 extern void slrn_quit (int);
@@ -91,12 +96,15 @@ extern FILE *Slrn_Debug_Fp;
 extern int slrn_sys_system (char *);
 
 extern void slrn_init_graphic_chars (void);
-extern SLsmg_Char_Type Graphic_LTee_Char;
-extern SLsmg_Char_Type Graphic_UTee_Char;
-extern SLsmg_Char_Type Graphic_LLCorn_Char;
-extern SLsmg_Char_Type Graphic_HLine_Char;
-extern SLsmg_Char_Type Graphic_VLine_Char;
-extern SLsmg_Char_Type Graphic_ULCorn_Char;
+#if SLANG_VERSION < 20000
+typedef char SLwchar_Type;
+#endif
+extern SLwchar_Type Graphic_LTee_Char;
+extern SLwchar_Type Graphic_UTee_Char;
+extern SLwchar_Type Graphic_LLCorn_Char;
+extern SLwchar_Type Graphic_HLine_Char;
+extern SLwchar_Type Graphic_VLine_Char;
+extern SLwchar_Type Graphic_ULCorn_Char;
 extern int Graphic_Chars_Mode;
 #define ALT_CHAR_SET_MODE	1
 #define SIMULATED_CHAR_SET_MODE	2
@@ -145,8 +153,7 @@ extern int Graphic_Chars_Mode;
 #define UNDERLINE_COLOR		31
 #define ITALICS_COLOR		32
 #define URL_COLOR		33
-#define QUOTE_COLOR	50
-
-#define MAX_QUOTE_LEVELS   8
+#define QUOTE_COLOR		50
 
+#define MAX_QUOTE_COLORS	 8
 #endif				       /* _SLRN_SLRN_H_ */
