@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -38,11 +38,22 @@
 #ifdef HAVE_STDLIB_H
 # include <stdlib.h>
 #endif
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
+
+#ifdef VMS
+# include "vms.h"
+#else
+# include <sys/types.h>
+# include <sys/stat.h>
+#endif
+
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
+#ifdef HAVE_SYS_FCNTL_H
+# include <sys/fcntl.h>
 #endif
 
 #include <ctype.h>
@@ -61,7 +72,7 @@
 #include "util.h"
 #include "snprintf.h"
 #if SLRN_USE_SLTCP
-# include "sltcp.c"
+/* # include "sltcp.c" */
 #else
 # include "clientlib.c"
 #endif
@@ -72,11 +83,8 @@
 #include "ranges.h"
 #include "vfile.h"
 
-#include "sortdate.c"
+/* #include "sortdate.c" */
 #include "score.c"
-#if SLRN_HAS_MIME
-# include "mime.c"
-#endif
 
 #include "xover.c"
 
@@ -84,12 +92,17 @@
 #define SLRN_HAS_MSGID_CACHE 1
 
 #include "hash.c"
+#include "common.h"
 
 /*}}}*/
 
 /*{{{ slrnpull global variables and structures */
 
-static char *Slrnpull_Version = SLRN_VERSION;
+static char *Slrnpull_Version_String = SLRN_VERSION_STRING;
+
+#if SLANG_VERSION >= 20000
+int Slrn_UTF8_Mode = 0;
+#endif
 
 #if SLRNPULL_USE_SETGID_POSTS
 # define OUTGOING_DIR_MODE	(0777 | 03000)
@@ -163,12 +176,12 @@ typedef struct _Active_Group_Type /*{{{*/
     * and a message id of 80 characters.  Of course that number would be
     * smaller if Pine were eliminated.
     */
-   unsigned int min, max;	       /* range of articles that slrnpull has 
+   NNTP_Artnum_Type min, max;	       /* range of articles that slrnpull has 
 					* already dealt with */
-   unsigned int active_min, active_max;/* article numbers that are in spool dir */
-   unsigned int server_min;	       /* artcle numbers that server reports */
-   unsigned int server_max;
-   
+   NNTP_Artnum_Type active_min, active_max;/* article numbers that are in spool dir */
+   NNTP_Artnum_Type server_min;	       /* artcle numbers that server reports */
+   NNTP_Artnum_Type server_max;
+
    
    unsigned int max_to_get;	       /* if non-zero, get only this many */
    unsigned int expire_days;	       /* if zero, no expiration */
@@ -225,7 +238,7 @@ static void va_log (FILE *fp, char *pre, char *fmt, va_list ap) /*{{{*/
 {
    char buf[2048];   
 
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    
    write_log (fp, pre, buf);
    
@@ -238,6 +251,7 @@ static void va_log (FILE *fp, char *pre, char *fmt, va_list ap) /*{{{*/
 
 /*}}}*/
 
+static void log_message (char *fmt, ...) SLATTRIBUTE_PRINTF(1,2);
 static void log_message (char *fmt, ...) /*{{{*/
 {
    va_list ap;
@@ -249,6 +263,7 @@ static void log_message (char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
+static void log_error (char *fmt, ...) SLATTRIBUTE_PRINTF(1,2);
 static void log_error (char *fmt, ...) /*{{{*/
 {
    va_list ap;
@@ -347,11 +362,11 @@ static FILE *open_server_min_file (Active_Group_Type *g, char *mode, /*{{{*/
 
 /*}}}*/
 
-static unsigned int read_server_min_file (Active_Group_Type *g) /*{{{*/
+static NNTP_Artnum_Type read_server_min_file (Active_Group_Type *g) /*{{{*/
 {
    char file[SLRN_MAX_PATH_LEN + 1];
    char line[256];
-   unsigned int retval;
+   NNTP_Artnum_Type retval;
    FILE *fp;
 
    fp = open_server_min_file (g, "r", file, sizeof (file));
@@ -368,7 +383,8 @@ static unsigned int read_server_min_file (Active_Group_Type *g) /*{{{*/
 
    fclose (fp);
    
-   if (1 != sscanf (line, "%u", &retval))
+   if ((1 != sscanf (line, NNTP_FMT_ARTNUM, &retval))
+       || (retval < 0))
      {
 	log_error (_("Error parsing %s."), file);
 	return 0;
@@ -378,7 +394,7 @@ static unsigned int read_server_min_file (Active_Group_Type *g) /*{{{*/
 }
 /*}}}*/
 
-static void write_server_min_file (Active_Group_Type *g, unsigned int val) /*{{{*/
+static void write_server_min_file (Active_Group_Type *g, NNTP_Artnum_Type val) /*{{{*/
 {
    char file[SLRN_MAX_PATH_LEN + 1];
    FILE *fp;
@@ -389,7 +405,7 @@ static void write_server_min_file (Active_Group_Type *g, unsigned int val) /*{{{
 	log_error (_("Unable to open %s for writing."), file);
 	return;
      }
-   if (EOF == fprintf (fp, "%u", val))
+   if (EOF == fprintf (fp, NNTP_FMT_ARTNUM, val))
      {
 	log_error (_("Write to %s failed."), file);
 	(void) fclose (fp);
@@ -421,7 +437,7 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
 {
    char file[SLRN_MAX_PATH_LEN + 1];
    char line[256];
-   unsigned int min, max;
+   NNTP_Artnum_Type min, max;
    FILE *fp;
 
    g->min = 1;
@@ -441,12 +457,17 @@ static int read_group_min_max_file (Active_Group_Type *g) /*{{{*/
 
    fclose (fp);
    
-   if (2 != sscanf (line, "%u %u", &min, &max))
+   if (2 != sscanf (line, NNTP_FMT_ARTNUM_2, &min, &max))
      {
 	log_error (_("Error parsing %s."), file);
 	return -1;
      }
-   
+   if ((min < 0) || (max < 0))
+     {
+	log_error (_("Expecting minmax numbers to be non-negative"));
+	return -1;
+     }
+
    g->active_min = g->min = min;
    g->server_max = g->active_max = g->max = max;
    
@@ -466,7 +487,7 @@ static int write_group_min_max_file (Active_Group_Type *g) /*{{{*/
 	log_error (_("Unable to open %s for writing."), file);
 	return -1;
      }
-   if (EOF == fprintf (fp, "%u %u", g->min, g->max))
+   if (EOF == fprintf (fp, NNTP_FMT_ARTNUM_2, g->min, g->max))
      {
 	log_error (_("Write to %s failed."), file);
 	(void) fclose (fp);
@@ -594,13 +615,13 @@ static int read_active_groups (void) /*{{{*/
    fp = fopen (Active_Groups_File, "r");
    if (fp == NULL)
      {
-	if (NULL == (fp = fopen (SYSCONFDIR "/" SLRNPULL_CONF, "r")))
+	if (NULL == (fp = fopen (SLRN_CONF_DIR "/" SLRNPULL_CONF, "r")))
 	  {
 	     log_error (_("Unable to open active groups file %s"), Active_Groups_File);
 	     return -1;
 	  }
 	else
-	  Active_Groups_File = SYSCONFDIR "/" SLRNPULL_CONF;
+	  Active_Groups_File = SLRN_CONF_DIR "/" SLRNPULL_CONF;
      }
    
    log_message (_("Reading %s"), Active_Groups_File);
@@ -797,7 +818,6 @@ static int make_filenames (void) /*{{{*/
    
    New_Groups_Time_File = data_dircat (New_Groups_Time_File);
    Active_File = data_dircat (Active_File);
-   Headers_File = data_dircat (Headers_File);
    New_Groups_File = data_dircat (New_Groups_File);
    
    Outgoing_Bad_Dir = slrn_spool_dircat (Outgoing_Dir, SLRNPULL_OUTGOING_BAD_DIR, 0);
@@ -814,7 +834,6 @@ static int make_filenames (void) /*{{{*/
      return -1;
    
    Overview_File = SLRN_SPOOL_NOV_FILE;
-   Headers_File = SLRN_SPOOL_HEADERS;
    Group_Min_Max_File = ".minmax";
    Server_Min_File = ".servermin";
 
@@ -823,10 +842,11 @@ static int make_filenames (void) /*{{{*/
 
 /*}}}*/
 
-static int *listgroup_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned int *nump) /*{{{*/
+static NNTP_Artnum_Type *listgroup_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned int *nump) /*{{{*/
 {
-   int *numbers;
-   unsigned int max, num, maxnum, curnum;
+   NNTP_Artnum_Type *numbers;
+   NNTP_Artnum_Type maxnum, curnum;
+   unsigned int max, num;
    char buf[256];
    int status;
    
@@ -847,10 +867,10 @@ static int *listgroup_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned int
      {
 	if (num == max)
 	  {
-	     int *newnums;
+	     NNTP_Artnum_Type *newnums;
 	     
 	     max += 1000;
-	     newnums = (int *) slrn_realloc ((char *) numbers, max * sizeof (int), 1);
+	     newnums = (NNTP_Artnum_Type *) slrn_realloc ((char *) numbers, max * sizeof (NNTP_Artnum_Type), 1);
 	     if (newnums == NULL)
 	       {
 		  slrn_free ((char *)numbers);
@@ -859,7 +879,7 @@ static int *listgroup_numbers (NNTP_Type *s, Active_Group_Type *g, unsigned int
 	       }
 	     numbers = newnums;
 	  }
-	curnum = atoi (buf);
+	curnum = NNTP_STR_TO_ARTNUM (buf);
 	if (curnum > maxnum) maxnum = curnum;
 	numbers[num] = curnum;
 	num++;
@@ -885,15 +905,16 @@ static unsigned int Num_Duplicates;
  * in group g on server s. *nump is set to the length of the array.
  * If article numbers larger than g->server_max are found, that variable
  * gets updated. */
-static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
-				 int server_min, int server_max,
-				 unsigned int *nump) /*{{{*/
+static NNTP_Artnum_Type *
+  list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
+		       NNTP_Artnum_Type server_min, NNTP_Artnum_Type server_max,
+		       unsigned int *nump) /*{{{*/
 {
    char *name; 
-   unsigned int min, max;
+   NNTP_Artnum_Type min, max;
    int status;
    char buf [512];
-   int *numbers;
+   NNTP_Artnum_Type *numbers;
    unsigned int num_numbers, max_num_numbers;
    
    *nump = 0;
@@ -911,16 +932,16 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
     * all the message-ids when the user specifies a limit to grab.
     */
    if ((max <= 1) && (g->max_to_get != 0)
-       && ((unsigned int) server_max-server_min+1 > g->max_to_get))
+       && (server_max + 1 > server_min + g->max_to_get))
      {
-	max = (unsigned int) server_max - g->max_to_get + 1;
+	max = server_max - g->max_to_get + 1;
 	if (server_max >= server_min)
-	  log_message (_("The server may contain as many as %d articles in this group"),
+	  log_message (_("The server may contain as many as " NNTP_FMT_ARTNUM  " articles in this group"),
 		       server_max - server_min + 1);
 	log_message (_("Only examining last %u articles since this is a new group"), g->max_to_get);
      }
 
-   status = nntp_server_vcmd (s, "XHDR Message-ID %u-", max);
+   status = nntp_server_vcmd (s, ("XHDR Message-ID " NNTP_FMT_ARTNUM "-"), max);
    if (status == -1)
      {
 	log_error (_("Server failed to return proper code for XHDR.  The connection may be lost")); 
@@ -945,41 +966,41 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
    min = max;
    while (1 == (status = nntp_read_line (s, buf, sizeof (buf))))
      {
-	int num;
+	NNTP_Artnum_Type num;
 	char *b1, *b2;
-	
-	num = (int) atoi (buf);
-	b1 = slrn_strchr (buf, '<');
+
+	num = NNTP_STR_TO_ARTNUM (buf);
+	b1 = slrn_strbyte (buf, '<');
 	if (b1 == NULL)
 	  {
 	     /* defective server?? */
 	     continue;
 	  }
-	b2 = slrn_strchr (b1, '>');
+	b2 = slrn_strbyte (b1, '>');
 	if (b2 == NULL) continue;
 	
 	b2++;
 	*b2 = 0;
 	
-	if (num > (int) max) max = (unsigned int) num;
+	if (num > max) max = num;
 
 	if (NULL != is_msgid_cached (b1, name, num, 0))
 	  {
 	     if (g->min > g->max) g->min = num;
 	     g->max = max;	       /* was: g->max = num */
 	     
-	     if (g->server_max < (unsigned int) num) g->server_max = num;
-	     
+	     if (g->server_max < num) g->server_max = num;
+
 	     Num_Duplicates++;
 	     continue;
 	  }
 	
 	if (num_numbers == max_num_numbers)
 	  {
-	     int *newnums;
+	     NNTP_Artnum_Type *newnums;
 	     
 	     max_num_numbers += 100;
-	     newnums = (int *) slrn_realloc ((char *) numbers, max_num_numbers * sizeof (int), 1);
+	     newnums = (NNTP_Artnum_Type *) slrn_realloc ((char *) numbers, max_num_numbers * sizeof (NNTP_Artnum_Type), 1);
 	     if (newnums == NULL)
 	       {
 		  slrn_free ((char *)numbers);
@@ -992,7 +1013,7 @@ static int *list_server_numbers (NNTP_Type *s, Active_Group_Type *g,
 	num_numbers++;
      }
 
-   log_message (_("%s: Retrieving %s %d-%d."), g->name,
+   log_message (_("%s: Retrieving %s " NNTP_FMT_ARTRANGE "."), g->name,
 		g->headers_only ? _("headers") : _("articles"),
 		min, max);
    if (num_numbers) g->server_max = max;
@@ -1092,7 +1113,7 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
 
 #if CREATE_OVERVIEW
    if ((EOF == fprintf (fp,
-			"%d\t%s\t%s\t%s\t%s\t%s\t%d\t%d",
+			(NNTP_FMT_ARTNUM "\t%s\t%s\t%s\t%s\t%s\t%d\t%d"),
 			xov->id, xov->subject_malloced, 
 			xov->from, xov->date_malloced, xov->message_id,
 			xov->references, xov->bytes, xov->lines))
@@ -1100,7 +1121,7 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
 	   && (EOF == fprintf (fp, "\tXref: %s", xov->xref)))
        || (EOF == fputc ('\n', fp)))
      {
-	log_error (_("Error writing to overview database: %s:%d."), Current_Group->name, xov->id);
+	log_error (_("Error writing to overview database:%s:" NNTP_FMT_ARTNUM), Current_Group->name, xov->id);
 	return -1;
      }
 #else
@@ -1111,13 +1132,13 @@ static int write_xover_line (FILE *fp, Slrn_XOver_Type *xov) /*{{{*/
 
 /*}}}*/
 
-static int append_body (Active_Group_Type *g, int n, char *body) /*{{{*/
+static int append_body (Active_Group_Type *g, NNTP_Artnum_Type n, char *body) /*{{{*/
 {
    char file[SLRN_MAX_PATH_LEN + 1];
-   char buf[128];
+   char buf[256];
    FILE *fp;
    
-   slrn_snprintf (buf, sizeof(buf), "%d", n);
+   (void) SLsnprintf (buf, sizeof(buf), NNTP_FMT_ARTNUM, n);
    
    if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
 			   file, sizeof (file)))
@@ -1155,12 +1176,12 @@ static int append_body (Active_Group_Type *g, int n, char *body) /*{{{*/
 }
 /*}}}*/
 
-static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
+static int write_head_and_body (Active_Group_Type *g, NNTP_Artnum_Type n, /*{{{*/
 				char *head, char *body, 
 				Slrn_XOver_Type *xov, FILE *xov_fp)
 {
    char file [SLRN_MAX_PATH_LEN + 1];
-   char buf[128];
+   char buf[256];
    FILE *fp;
    
    if (head == NULL)
@@ -1170,9 +1191,9 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
 	
 	return 0;
      }
-   
-   slrn_snprintf (buf, sizeof (buf), "%d", n);
-   
+
+   (void) SLsnprintf (buf, sizeof (buf), NNTP_FMT_ARTNUM, n);
+
    if ((-1 == slrn_dircat (SlrnPull_Spool_News_Dir, g->dirname,
 			   file, sizeof (file)))
        || (-1 == slrn_dircat (file, buf, file, sizeof (file))))
@@ -1240,7 +1261,7 @@ static int fetch_body (NNTP_Type *s, char **body) /*{{{*/
 
 /*}}}*/
 
-static int get_bodies (NNTP_Type *s, int *numbers, /*{{{*/
+static int get_bodies (NNTP_Type *s, NNTP_Artnum_Type *numbers, /*{{{*/
 		       char **heads, char **bodies, unsigned int num)
 {
    unsigned int i;
@@ -1257,8 +1278,8 @@ static int get_bodies (NNTP_Type *s, int *numbers, /*{{{*/
 	if (heads[i] == NULL)
 	  continue;
 	
-	slrn_snprintf (b, sizeof (buf) - (size_t)(b - buf),
-		       "%sbody %d", crlf, numbers[i]);
+	(void) SLsnprintf (b, sizeof (buf) - (size_t)(b - buf),
+			   ("%sbody " NNTP_FMT_ARTNUM), crlf, numbers[i]);
 	b += strlen (b);
 	
 	crlf = "\r\n";
@@ -1284,7 +1305,20 @@ static int get_bodies (NNTP_Type *s, int *numbers, /*{{{*/
 
 /*}}}*/
 
-static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **headers, Slrn_XOver_Type *xov) /*{{{*/
+static void free_header_data (Slrn_Header_Type *h)
+{
+   slrn_free (h->tree_ptr);
+   slrn_free (h->subject);
+   slrn_free (h->from);
+   slrn_free (h->date);
+   slrn_free (h->msgid);
+   slrn_free (h->refs);
+   slrn_free (h->xref);
+   slrn_free (h->realname);
+   slrn_free_additional_headers (h->add_hdrs);
+}
+
+static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, NNTP_Artnum_Type n, char **headers, Slrn_XOver_Type *xov) /*{{{*/
 {
    int status;
    Slrn_Header_Type h;
@@ -1292,6 +1326,7 @@ static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **header
    Slrn_Score_Debug_Info_Type *sdi = NULL;
    
    *headers = NULL;
+   memset ((char *) &h, 0, sizeof (Slrn_Header_Type));
 
    print_time_stats (s, 0);
 
@@ -1313,20 +1348,15 @@ static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **header
 	return 0;
      }
    
-   slrn_map_xover_to_header (xov, &h);
+   slrn_map_xover_to_header (xov, &h, 0);
 
-#if SLRN_HAS_MIME
-   h.subject = slrn_safe_strmalloc (xov->subject_malloced);
-   h.from = slrn_safe_strmalloc (xov->from);
-   if (Slrn_Use_Mime)
-     {
-	slrn_rfc1522_decode_string (h.subject);
-	slrn_rfc1522_decode_string (h.from);
-     }
+#if 0
+   /* FIXME -- I need to think more about this, and check for errors */
+   slrn_rfc1522_decode_string (&h.subject);
+   slrn_rfc1522_decode_string (&h.from);
 #endif
-   
 #if 1
-   (void) is_msgid_cached (h.msgid, Current_Group->name, (unsigned int) n, 1);
+   (void) is_msgid_cached (h.msgid, Current_Group->name, n, 1);
 #endif
 
    score = slrn_score_header (&h, Current_Group->name, (KLog_Fp != NULL) ? &sdi : NULL);
@@ -1364,18 +1394,16 @@ static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **header
 	  g->requests = slrn_ranges_add (g->requests, n, n);
 /*	return 0; */
      }
+
    while (sdi != NULL)
      {
 	Slrn_Score_Debug_Info_Type *hlp = sdi->next;
 	slrn_free ((char *)sdi);
 	sdi = hlp;
      }
-#if SLRN_HAS_MIME
-   slrn_free (h.subject);
-   h.subject = xov->subject_malloced;
-   slrn_free (h.from);
-   h.from = xov->from;
-#endif
+
+   free_header_data (&h);
+
 #if 0
    /* This next call should add the message id to the cache. */
    (void) is_msgid_cached (h.msgid, Current_Group->name, (unsigned int) n, 1);
@@ -1385,7 +1413,7 @@ static int fetch_head (NNTP_Type *s,  Active_Group_Type *g, int n, char **header
 
 /*}}}*/
 
-static int get_heads (NNTP_Type *s,  Active_Group_Type *g, int *numbers, char **heads, /*{{{*/
+static int get_heads (NNTP_Type *s,  Active_Group_Type *g, NNTP_Artnum_Type *numbers, char **heads, /*{{{*/
 		      Slrn_XOver_Type *xovs, unsigned int num)
 {
    unsigned int i;
@@ -1402,7 +1430,7 @@ static int get_heads (NNTP_Type *s,  Active_Group_Type *g, int *numbers, char **
 	unsigned int len = sizeof (buf) - (size_t) (b - buf);
 	if (len < 20)
 	  slrn_exit_error (_("Internal error: Buffer in get_heads not large enough!"));
-	slrn_snprintf (b, len, "%shead %d", crlf, numbers[i]);
+	(void) SLsnprintf (b, len, "%shead " NNTP_FMT_ARTNUM, crlf, numbers[i]);
 	crlf = "\r\n";
 	b += strlen (b);
 	
@@ -1455,9 +1483,9 @@ static void get_marked_bodies (NNTP_Type *s, Active_Group_Type *g) /*{{{*/
 {
    char *heads[SLRN_MAX_QUEUED];
    char *bodies[SLRN_MAX_QUEUED];
-   int numbers[SLRN_MAX_QUEUED];
+   NNTP_Artnum_Type numbers[SLRN_MAX_QUEUED];
    unsigned int i, num;
-   int max, bmin, bmax;
+   NNTP_Artnum_Type max, bmin, bmax;
    Slrn_Range_Type *r;
 
    Num_Articles_Received = 0;
@@ -1480,7 +1508,7 @@ static void get_marked_bodies (NNTP_Type *s, Active_Group_Type *g) /*{{{*/
      {
 	heads[i] = ""; /* insert dummy headers so get_bodies won't skip them */
      }
-   
+
    max = r->min;
    while (r != NULL)
      {
@@ -1545,7 +1573,7 @@ static void get_marked_bodies (NNTP_Type *s, Active_Group_Type *g) /*{{{*/
 }
 /*}}}*/
 
-static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsigned int num) /*{{{*/
+static int get_articles (NNTP_Type *s, Active_Group_Type *g, NNTP_Artnum_Type *numbers, unsigned int num) /*{{{*/
 {
    unsigned int i;
 #ifndef SLRN_MAX_QUEUED
@@ -1590,7 +1618,7 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 
    if (g->headers_only) /* "register" new articles without body */
      {
-	int bmin, bmax;
+	NNTP_Artnum_Type bmin, bmax;
 	unsigned int j=0;
 	
 	while ((j < num) && (heads[j]==NULL))
@@ -1618,9 +1646,7 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 	if (!g->headers_only)
 	  slrn_free (bodies[i]);
 	slrn_free (heads[i]);
-	slrn_free (xovs[i].subject_malloced);
-	slrn_free (xovs[i].date_malloced);
-	slrn_free_additional_headers (xovs[i].add_hdrs);
+	slrn_free_xover_data (xovs+i);
      }
    return ret;
 }
@@ -1628,12 +1654,12 @@ static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsig
 /*}}}*/
 
 static int get_group_articles (NNTP_Type *s, Active_Group_Type *g, 
-			       int server_min, int server_max, int marked_bodies) /*{{{*/
+			       NNTP_Artnum_Type server_min, NNTP_Artnum_Type server_max, int marked_bodies) /*{{{*/
 {
    if (!marked_bodies)
      {
-	unsigned int gmin, gmax;
-	int *numbers;
+	NNTP_Artnum_Type gmin, gmax;
+	NNTP_Artnum_Type *numbers;
 	unsigned int num_numbers, i, imin;
 	
 	Num_Articles_Received = 0;
@@ -1642,12 +1668,12 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
 	
 	gmin = g->min;
 	gmax = g->max;
-	
+
 	if (((server_min > server_max) || (server_max < 0))
-	    || (((unsigned int)server_max <= gmax) && (gmin <= gmax)))
+	    || ((server_max <= gmax) && (gmin <= gmax)))
 	  {
 	     log_message (_("%s: no new articles available."), g->name);
-	     return 0;
+	     goto fetch_marked_bodies;
 	  }
 	
 	Num_Duplicates = 0;
@@ -1659,7 +1685,7 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
 	if (numbers == NULL) return -1;
 	
 	i = 0;
-	while ((i < num_numbers) && (numbers[i] <= (int) gmax))
+	while ((i < num_numbers) && (numbers[i] <= gmax))
 	  i++;
 	
 	if (i == num_numbers)
@@ -1667,7 +1693,7 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
 	     g->max = g->server_max;
 	     log_message (_("%s: No new articles available."), g->name);
 	     slrn_free ((char *) numbers);
-	     return 0;
+	     goto fetch_marked_bodies;
 	  }
 	
 	log_message (_("%s: %u articles available."), g->name, num_numbers - i);
@@ -1689,7 +1715,7 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
 	
 	while (i < num_numbers)
 	  {
-	     int ns[SLRN_MAX_QUEUED];
+	     NNTP_Artnum_Type ns[SLRN_MAX_QUEUED];
 	     unsigned int j;
 	     
 	     j = 0;
@@ -1716,7 +1742,8 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
      }
 
    /* Now, fetch marked article bodies. */
-   
+
+   fetch_marked_bodies:
    /* Don't request bodies that are no longer there. */
    if (server_min > 1)
      g->requests = slrn_ranges_remove (g->requests, 1, server_min-1);
@@ -1735,7 +1762,7 @@ static int pull_news (NNTP_Type *s, int marked_bodies) /*{{{*/
    g = Active_Groups;
    while (g != NULL)
      {
-	int min, max;
+	NNTP_Artnum_Type min, max;
 
 	log_message (_("Fetching articles for %s."), g->name);
 	
@@ -1753,7 +1780,7 @@ static int pull_news (NNTP_Type *s, int marked_bodies) /*{{{*/
 	  }
 
 	write_server_min_file (g, min);
-	if (g->server_max > (unsigned int) max)
+	if (g->server_max > max)
 	  g->server_max = max;
 
 	Current_Group = g;
@@ -1926,7 +1953,7 @@ static int write_active (void) /*{{{*/
    FILE *fp;
    char file [SLRN_MAX_PATH_LEN + 5];
    
-   slrn_snprintf (file, sizeof (file), "%s.tmp", Active_File);
+   (void) SLsnprintf (file, sizeof (file), "%s.tmp", Active_File);
    
    fp = fopen (file, "w");
    if (fp == NULL)
@@ -1937,7 +1964,7 @@ static int write_active (void) /*{{{*/
    
    while (g != NULL)
      {
-	if (EOF == fprintf (fp, "%s %u %u y\n", g->name, 
+	if (EOF == fprintf (fp, ("%s " NNTP_FMT_ARTNUM_2 " y\n"), g->name,
 			    g->active_max, g->active_min))
 	  {
 	     fclose (fp);
@@ -1983,6 +2010,7 @@ static void connection_lost_hook (NNTP_Type *s)
 static int open_servers (char *host) /*{{{*/
 {
 #if SLRN_USE_SLTCP
+   SLtcp_Verror_Hook = va_log_error;
    if (-1 == sltcp_open_sltcp ())
      {
 	fprintf (stderr, _("Error opening sltcp interface.\n"));
@@ -2136,7 +2164,7 @@ static void usage (char *pgm) /*{{{*/
 
 static void show_version (char *pgm)
 {
-   log_error (_("%s version: %s\n"), pgm, Slrnpull_Version);
+   log_error (_("%s version: %s\n"), pgm, Slrnpull_Version_String);
    close_log_files ();
    exit (0);
 }
@@ -2180,8 +2208,15 @@ int main (int argc, char **argv) /*{{{*/
    (void) setlocale(LC_ALL, "");
 #endif
 #ifdef ENABLE_NLS
-   bindtextdomain(PACKAGE,LOCALEDIR);
-   textdomain(PACKAGE);
+   bindtextdomain(NLS_PACKAGE_NAME, NLS_LOCALEDIR);
+   textdomain (NLS_PACKAGE_NAME);
+#endif
+
+#if SLANG_VERSION >= 20000
+   Slrn_UTF8_Mode = SLutf8_enable (-1);
+   if (Slrn_UTF8_Mode == 0)
+     SLsmg_utf8_enable (1);
+   Slrn_UTF8_Mode = 1;
 #endif
    
    pgm = argv[0];
@@ -2253,7 +2288,19 @@ int main (int argc, char **argv) /*{{{*/
 	  }
 	else if (!strcmp (arg, "--debug") && (argc > 0))
 	  {
-	     if (NULL == (Slrn_Debug_Fp = fopen (*argv, "w")))
+	     int fd;
+	     if (Slrn_Debug_Fp != NULL)
+	       {
+		  fclose (Slrn_Debug_Fp);
+		  Slrn_Debug_Fp = NULL;
+	       }
+	     fd = open (*argv, O_WRONLY | O_CREAT, S_IREAD | S_IWRITE);
+	     if (fd != -1)
+	       {
+		  if (NULL == (Slrn_Debug_Fp = fdopen (fd, "w")))
+		    close (fd);
+	       }
+	     if (Slrn_Debug_Fp == NULL)
 	       {
 		  fprintf (stderr, _("Unable to create debug file %s\n"), *argv);
 		  exit (1);
@@ -2618,7 +2665,24 @@ int slrn_message (char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
+#if 0
+/* For now, we need this to link against charset.o
+ * Maybe seek a cleaner alternative later. */
 
+static void slrn_art_free_article_line (Slrn_Article_Line_Type *l)
+{
+   Slrn_Article_Line_Type *next;
+   
+   while (l != NULL)
+     {
+	slrn_free ((char *) l->buf);
+	next = l->next;
+	slrn_free ((char *) l);
+	l = next;
+     }
+}   
+
+#endif
 /*}}}*/
 
 static char *read_header_from_file (char *file, int *has_body)
@@ -2669,7 +2733,7 @@ static char *read_header_from_file (char *file, int *has_body)
    return mbuf;
 }
 
-static int sort_int_cmp (unsigned int *a, unsigned int *b)
+static int sort_artnums_cmp (NNTP_Artnum_Type *a, NNTP_Artnum_Type *b)
 {
    if (*a > *b) return 1;
    if (*a == *b) return 0;
@@ -2680,16 +2744,16 @@ static int sort_int_cmp (unsigned int *a, unsigned int *b)
  * write an overview-file entry for a single article
  * returns -1 on error, 0 if header is missing, 1 otherwise
  */
-static int write_overview_entry(FILE *xov_fp, int id, char *dir)
+static int write_overview_entry(FILE *xov_fp, NNTP_Artnum_Type id, char *dir)
 {
    char file [SLRN_MAX_PATH_LEN + 1];
    char *header;
    struct stat st;
    Slrn_XOver_Type xov;
    int has_body;
-   char buf[32];
+   char buf[256];
    
-   sprintf (buf, "%d", id); /* safe */
+   sprintf (buf, NNTP_FMT_ARTNUM, id); /* safe */
    
    if (-1 == slrn_dircat (dir, buf, file, sizeof (file)))
      {
@@ -2709,13 +2773,13 @@ static int write_overview_entry(FILE *xov_fp, int id, char *dir)
    header = read_header_from_file (file, &has_body);
    if (header == NULL)
      {
-        log_error(_("Unable to read header %d in %s."), id, file);
+        log_error(_("Unable to read header " NNTP_FMT_ARTNUM " in %s."), id, file);
         return -1;
      }
    
    if (-1 == xover_parse_head (id, header, &xov))
      {
-        log_error(_("Header %d not parseable for overview."), id);
+        log_error(_("Header " NNTP_FMT_ARTNUM " not parseable for overview."), id);
         slrn_free (header);
         return -1;
      }
@@ -2723,13 +2787,11 @@ static int write_overview_entry(FILE *xov_fp, int id, char *dir)
    if (-1 == write_xover_line (xov_fp, &xov))
      {
         slrn_free (header);
-        slrn_free (xov.subject_malloced);
-        slrn_free (xov.date_malloced);
+	slrn_free_xover_data (&xov);
         return -1;
      }
    slrn_free (header);
-   slrn_free (xov.subject_malloced);
-   slrn_free (xov.date_malloced);
+   slrn_free_xover_data (&xov);
    return has_body;
 }
 
@@ -2747,14 +2809,14 @@ static void progress_update_overview(int i, int n_nums, Active_Group_Type *g, in
 }
 
 /* Now also updates g->headers */
-static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, unsigned int n_nums)
+static int create_overview_for_dir (Active_Group_Type *g, NNTP_Artnum_Type *nums, unsigned int n_nums)
 {
    char dir [SLRN_MAX_PATH_LEN + 1];
    FILE *xov_fp;
    unsigned int i;
-   int bmin, bmax;
+   NNTP_Artnum_Type bmin, bmax;
    Slrn_Range_Type *headers = NULL;
-   void (*qsort_fun) (char *, unsigned int, int, int (*)(unsigned int *, unsigned int *));
+   void (*qsort_fun) (char *, unsigned int, int, int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type *));
 
    log_message (_("Creating Overview file for %s..."), g->name);
    
@@ -2769,12 +2831,12 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
    
    if ((nums != NULL) && (n_nums != 0))
      {
-	qsort_fun = (void (*)(char *, unsigned int, int, 
-			      int (*)(unsigned int *, unsigned int *))) 
+	qsort_fun = (void (*)(char *, unsigned int, int,
+			      int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type *)))
 	  qsort;
-	
-	(*qsort_fun) ((char *) nums, n_nums, sizeof (unsigned int), sort_int_cmp);
-	
+
+	(*qsort_fun) ((char *) nums, n_nums, sizeof (NNTP_Artnum_Type), sort_artnums_cmp);
+
 	g->active_min = g->min = nums[0];
 	g->active_max = nums [n_nums - 1];
      }
@@ -2783,12 +2845,12 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	g->active_min = g->min = g->max + 1;
 	g->active_max = g->active_min - 1;
      }
-   
+
    bmin = bmax = -1;
    for (i = 0; i < n_nums; i++)
      {
 	int ret;
-        if (-1 == (ret = write_overview_entry(xov_fp, (int) nums [i], dir)))
+        if (-1 == (ret = write_overview_entry(xov_fp, nums [i], dir)))
           {
              fclose(xov_fp);
 	     slrn_ranges_free (headers);
@@ -2798,7 +2860,7 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	  {
 	     bmin = bmax = nums[i];
 	  }
-	else if ((bmin != -1) && (ret || ((int)nums[i] != bmax+1)))
+	else if ((bmin != -1) && (ret || (nums[i] != bmax+1)))
 	  {
 	     headers = slrn_ranges_add (headers, bmin, bmax);
 	     if (ret)
@@ -2825,16 +2887,16 @@ static int create_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
  * which is a lot faster than recreating the entire file. If the overview
  * file cannot be opened for reading, a new one is created.
  */
-static int update_overview_for_dir (Active_Group_Type *g, unsigned int *nums, unsigned int n_nums)
+static int update_overview_for_dir (Active_Group_Type *g, NNTP_Artnum_Type *nums, unsigned int n_nums)
 {
    char file [SLRN_MAX_PATH_LEN + 1];
    char newfile [SLRN_MAX_PATH_LEN + 1];
    char dir [SLRN_MAX_PATH_LEN + 1];
    char buf [4096];
-   unsigned int xov_nr = 0;
+   NNTP_Artnum_Type xov_nr = 0;
    FILE *xov_fp, *new_xov_fp;
    unsigned int i;
-   void (*qsort_fun) (char *, unsigned int, int, int (*)(unsigned int *, unsigned int *));
+   void (*qsort_fun) (char *, unsigned int, int, int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type *));
    
    xov_fp = open_xover_file (g, "r");
    if (xov_fp == NULL)
@@ -2862,10 +2924,10 @@ static int update_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
    if ((nums != NULL) && (n_nums != 0))
      {
 	qsort_fun = (void (*)(char *, unsigned int, int, 
-			      int (*)(unsigned int *, unsigned int *))) 
+			      int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type *)))
 	  qsort;
-	
-	(*qsort_fun) ((char *) nums, n_nums, sizeof (unsigned int), sort_int_cmp);
+
+	(*qsort_fun) ((char *) nums, n_nums, sizeof (NNTP_Artnum_Type), sort_artnums_cmp);
 	
 	g->active_min = g->min = nums[0];
 	g->active_max = nums [n_nums - 1];
@@ -2890,13 +2952,13 @@ static int update_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	while (xov_nr < nums[i]) /* drop old overview entries */
 	  {
 	     if (NULL != fgets (buf, sizeof(buf), xov_fp))
-	       xov_nr = atoi (buf);
+	       xov_nr = NNTP_STR_TO_ARTNUM (buf);
 	     else
 	       /* end of overview file found, write remaining entries */
 	       {
 		  while (i < n_nums)
 		    {
-		       if (-1 == write_overview_entry (new_xov_fp, (int) nums [i], dir))
+		       if (-1 == write_overview_entry (new_xov_fp, nums [i], dir))
 			 break;
 		       
 		       if ((Stdout_Is_TTY) && (((i % 100) == 0) || (i+1 == n_nums)))
@@ -2921,7 +2983,7 @@ static int update_overview_for_dir (Active_Group_Type *g, unsigned int *nums, un
 	/* In case entries are missing, insert as many as needed */
 	while (nums [i] < xov_nr)
 	  {
-	     if (-1 == write_overview_entry (new_xov_fp, (int) nums [i], dir))
+	     if (-1 == write_overview_entry (new_xov_fp, nums[i], dir))
 	       goto end_of_loop; /* break 2 */
 	     if ((Stdout_Is_TTY) && (((i % 100) == 0) || (i+1 == n_nums)))
 	       progress_update_overview(i, n_nums, g, 0);
@@ -2957,12 +3019,11 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
    Slrn_Dirent_Type *df;
    char dir [SLRN_MAX_PATH_LEN + 1];
    char file [SLRN_MAX_PATH_LEN + 1];
-   unsigned int *ok_names;
-   unsigned int num_ok_names, max_num_ok_names;
+   NNTP_Artnum_Type *ok_names;
+   unsigned int i, n, num_ok_names, max_num_ok_names;
    unsigned int num_expired = 0;
-   unsigned int i, n;
    unsigned int left, right, cutoff;
-   void (*qsort_fun) (char *, unsigned int, int, int (*)(unsigned int *, unsigned int *));
+   void (*qsort_fun) (char *, unsigned int, int, int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type *));
    time_t expire_time;
    int perform_expire = 1;
    
@@ -2987,21 +3048,23 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
    while (NULL != (df = slrn_read_dir (dp)))
      {
 	char *name, *p;
-	
+	NNTP_Artnum_Type id;
+
 	name = df->name;
 	
 	/* Look for names composed of digits.  Skip others. */
 	p = name;
 	while (*p && isdigit (*p)) p++;
 	if (*p != 0) continue;
-	
-	if (1 != sscanf (name, "%u", &n))
+
+	if ((1 != sscanf (name, NNTP_FMT_ARTNUM, &id))
+	    || (id < 0))
 	  continue;		       /* hmm... I'm paranoid. */
 
 	if (num_ok_names == max_num_ok_names)
 	  {
 	     max_num_ok_names += 500;
-	     ok_names = (unsigned int *) slrn_realloc ((char *) ok_names, max_num_ok_names * sizeof (unsigned int), 1);
+	     ok_names = (NNTP_Artnum_Type *) slrn_realloc ((char *) ok_names, max_num_ok_names * sizeof (NNTP_Artnum_Type), 1);
 	     if (ok_names == NULL)
 	       {
 		  log_error (_("malloc error. Unable to expire group %s."), g->name);
@@ -3009,11 +3072,11 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
 		  return -1;
 	       }
 	  }
-	
-	ok_names [num_ok_names] = n;
+
+	ok_names [num_ok_names] = id;
 	num_ok_names++;
      }
-   
+
    slrn_close_dir (dp);
 
    if (num_ok_names==0) /* nothing to do */
@@ -3022,10 +3085,10 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
    /* Now, sort the array and use a binary search to find the lowest article
     * number we don't want to expire.
     */
-   qsort_fun = (void (*)(char *, unsigned int, int, 
-			 int (*)(unsigned int *, unsigned int *))) qsort;
+   qsort_fun = (void (*)(char *, unsigned int, int,
+			 int (*)(NNTP_Artnum_Type *, NNTP_Artnum_Type*))) qsort;
    
-   (*qsort_fun) ((char *) ok_names, num_ok_names, sizeof (unsigned int), sort_int_cmp);
+   (*qsort_fun) ((char *) ok_names, num_ok_names, sizeof (NNTP_Artnum_Type), sort_artnums_cmp);
 
    time (&expire_time);
    expire_time -= g->expire_days * (24 * 60 * 60);   
@@ -3040,13 +3103,13 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
 	 * article we want to keep. */
 	while (left < right)
 	  {
-	     char buf[32];
+	     char buf[256];
 	     struct stat st;
 	     
 	     cutoff = (left + right) / 2;
-	     
-	     sprintf (buf, "%d", ok_names[cutoff]); /* safe */
-	     
+
+	     sprintf (buf, NNTP_FMT_ARTNUM, ok_names[cutoff]); /* safe */
+
 	     if (-1 == slrn_dircat (dir, buf, file, sizeof (file)))
 	       goto stat_error;
 	     
@@ -3081,39 +3144,40 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
    if (cutoff < num_ok_names)
      {
 	Slrn_Range_Type *r;
-	unsigned int server_min = read_server_min_file (g);
+	NNTP_Artnum_Type id;
+	NNTP_Artnum_Type server_min = read_server_min_file (g);
 
-	i = ok_names[cutoff];
-	if (i>1)
-	  g->headers = slrn_ranges_remove (g->headers, 1, i-1);
+	id = ok_names[cutoff];
+	if (id>1)
+	  g->headers = slrn_ranges_remove (g->headers, 1, id-1);
 	r = g->headers;
 	
 	if (r!=NULL)
 	  {
-	     if ((int)i < r->min)
-	       i = r->min;
+	     if (id < r->min)
+	       id = r->min;
 	     
 	     n = cutoff;
-	     while (i < server_min)
+	     while (id < server_min)
 	       {
-		  while ((n<num_ok_names) && (ok_names[n]<i))
+		  while ((n<num_ok_names) && (ok_names[n]<id))
 		    n++;
 		  
-		  if ((n<num_ok_names) && (ok_names[n]==i))
+		  if ((n<num_ok_names) && (ok_names[n]==id))
 		    {
 		       ok_names[n] = ok_names[cutoff];
-		       ok_names[cutoff] = i;
+		       ok_names[cutoff] = id;
 		       cutoff++;
 		    }
 		  
-		  i++;
-		  if ((int)i>r->max)
+		  id++;
+		  if (id>r->max)
 		    {
 		       r = r->next;
-		       if (r != NULL)
-			 i = r->min;
-		       else
+		       if (r == NULL)
 			 break;
+
+		       id = r->min;
 		    }
 	       }
 	  }
@@ -3125,8 +3189,8 @@ static int expire_group (Active_Group_Type *g, int rebuild) /*{{{*/
    /* Now, actually delete the files on disk. */
    for (i=0; i < cutoff; i++)
      {
-	char buf[32];
-	sprintf (buf, "%d", ok_names[i]); /* safe */
+	char buf[256];
+	sprintf (buf, NNTP_FMT_ARTNUM, ok_names[i]); /* safe */
 	
 	if ((-1 == slrn_dircat (dir, buf, file, sizeof (file))) ||
 	    (-1 == slrn_delete_file (file)))
@@ -3185,14 +3249,25 @@ static int do_expire (int rebuild) /*{{{*/
 static char *Auth_User_Name;
 static char *Auth_Password;
 
-static int get_authorization (char *host, char **name, char **pass)
+static int get_authorization (char *host, int isreq, char **name, char **pass)
 {
+   int has_null_vals;
+
    (void) host;
 
+   *name = *pass = NULL;
+
+   has_null_vals = ((Auth_Password == NULL) || (Auth_User_Name == NULL));
+   if ((isreq == 0) && has_null_vals)
+     return 0;
+
+   if (has_null_vals)
+     return -1;
+
    *name = Auth_User_Name;
    *pass = Auth_Password;
 
-   return 0;
+   return 1;
 }
 
 static int read_authinfo (void)
@@ -3273,7 +3348,10 @@ static int read_headers_files (void)
 	
 	if (NULL != (vline = vgets (vp, &vlen)))
 	  {
-	     vline[vlen] = 0; /* make sure line is NULL terminated */
+	     if (vline[vlen-1] == '\n')
+	       vline[vlen-1] = 0; /* make sure line is NULL terminated */
+	     else
+	       vline[vlen] = 0;
 	     group->headers = slrn_ranges_from_newsrc_line (vline);
 	  }
 	
@@ -3382,7 +3460,10 @@ static void read_requests_files (void)
 	     if (NULL == (group = find_group_type (vline)))
 	       continue;
 	     
-	     vline[vlen-1] = 0;	       /* kill \n and NULL terminate */
+	     if (vline[vlen-1] == '\n')
+	       vline[vlen-1] = 0;
+	     else
+	       vline[vlen] = 0;
 	     
 	     r = slrn_ranges_from_newsrc_line (p+1);
 	     group->requests = slrn_ranges_merge (group->requests, r);
