@@ -2,8 +2,8 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
- Copyright (c) 2001-2003  Thomas Schultz <tststs@gmx.de>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
+ Copyright (c) 2001-2006  Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -57,7 +57,8 @@
 # define USE_WINSOCK_SLTCP	1
 #else
 # if defined(__WIN32__)
-#  define Win32_Winsock
+/* #  define Win32_Winsock */
+#  define __USE_W32_SOCKETS
 #  include <windows.h>
 #  define USE_WINSOCK_SLTCP	1
 # endif
@@ -113,9 +114,13 @@ extern int h_errno;
 # include <gnutls/openssl.h>
 #else
 # if SLTCP_HAS_SSL_SUPPORT
-#  include <openssl/ssl.h>
-#  include <openssl/err.h>
-#  include <openssl/rand.h>
+#  if SLTCP_HAS_NSS_COMPAT
+#   include <nss_compat_ossl.h>
+#  else
+#   include <openssl/ssl.h>
+#   include <openssl/err.h>
+#   include <openssl/rand.h>
+#  endif
 # endif
 #endif
 
@@ -162,6 +167,28 @@ int SLtcp_TimeOut_Secs = SLRN_SLTCP_TIMEOUT_SECS;
  * to get translated. */
 static int TCP_Verbose_Reporting = 0;
 
+void (*SLtcp_Verror_Hook) (char *, va_list);
+static void print_error (char *fmt, ...) SLATTRIBUTE_PRINTF(1,2);
+static void print_error (char *fmt, ...) 
+{
+   va_list ap;
+   
+   va_start (ap, fmt);
+   if (SLtcp_Verror_Hook != NULL)
+     (*SLtcp_Verror_Hook) (fmt, ap);
+   else
+     {
+	(void) vfprintf(stderr, fmt, ap);
+	(void) fputc ('\n', stderr);
+	fflush (stderr);
+     }
+   va_end (ap);
+}
+
+/* void (*SLtcp_Error_Hook) (char *, ...) SLATTRIBUTE_PRINTF(1,2); */
+
+
+
 static int sys_call_interrupted_hook (void) /*{{{*/
 {
    if (SLTCP_Interrupt_Hook == NULL)
@@ -205,7 +232,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
       snprintf(portstr, 6, "%i", port);
       if ((r = getaddrinfo(host, portstr, &hint, &res)) != 0) {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Error resolving %s (port %s): %s\n", host, portstr, gai_strerror(r));
+	    print_error ("Error resolving %s (port %s): %s\n", host, portstr, gai_strerror(r));
 	 }
 	 if (r == EAI_AGAIN) {
 	    slrn_sleep (1);
@@ -214,11 +241,11 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    } while (r && (tries++ <= 3));
    
    if (r) {
-      fprintf (stderr, _("Failed to resolve %s\n"), host);
+      print_error (_("Failed to resolve %s\n"), host);
       return -1;
    }
    if (TCP_Verbose_Reporting) {
-      fprintf (stderr, "Successfully resolved %s\n", host);
+      print_error ("Successfully resolved %s\n", host);
    }
 
    ai = res;
@@ -231,44 +258,45 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 	 int l;
 	 if (a->sa_family == AF_INET) {
 	    l = sizeof(struct sockaddr_in);
-	    fprintf (stderr, "Address family: AF_INET\n");
+	    print_error ("Address family: AF_INET\n");
 	 } else {
 	    assert(a->sa_family == AF_INET6);
 	    l = sizeof(struct sockaddr_in6);
-	    fprintf (stderr, "Address family: AF_INET6\n");
+	    print_error ("Address family: AF_INET6\n");
 	 }
 	 if (!getnameinfo(a, l, buf, NI_MAXHOST-1, NULL, 0, NI_NUMERICHOST)) {
-	    fprintf (stderr, "Will try with address %s", buf);
+	    print_error ("Will try with address %s", buf);
 	 } else {
-	    fprintf (stderr, "getnameinfo failed: %s\n", strerror(errno));
+	    print_error ("getnameinfo failed: %s\n", strerror(errno));
 	 }
 # else
 	 if (a->sa_family == AF_INET) {
-	    fprintf (stderr, "Address family: AF_INET\n");
+	    print_error ("Address family: AF_INET\n");
 	 } else {
 	    assert(a->sa_family == AF_INET6);
-	    fprintf (stderr, "Address family: AF_INET6\n");
+	    print_error ("Address family: AF_INET6\n");
 	 }
 # endif /* HAVE_GETNAMEINFO */
       }
       
       if ((fd = socket(ai->ai_family, SOCK_STREAM, 0)) == -1) {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Error creating socket: %s\n", strerror(errno));
+	    print_error ("Error creating socket: %s\n", strerror(errno));
 	 }
       } else {
 	 if (TCP_Verbose_Reporting) {
-	    fprintf (stderr, "Created socket; descriptor is = %i\n", fd);
+	    print_error ("Created socket; descriptor is = %i\n", fd);
 	 }
 	 if ((r = connect(fd, ai->ai_addr, ai->ai_addrlen)) == 0) {
 	    if (TCP_Verbose_Reporting) {
-	       fprintf (stderr, "Successfully connected\n");
+	       print_error ("Successfully connected\n");
 	    }
 	    connected = 1;
 	 } else {
 	    if (TCP_Verbose_Reporting) {
-	       fprintf (stderr, "Error connecting: %i, %s\n", errno, strerror(errno));
+	       print_error ("Error connecting: %i, %s\n", errno, strerror(errno));
 	    }
+	    (void) SLTCP_CLOSE (fd);
 	 }
       }
       ai = ai->ai_next;
@@ -277,7 +305,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    freeaddrinfo(res);
    
    if (!connected)
-     fprintf(stderr, _("Unable to make connection. Giving up.\n"));
+     print_error(_("Unable to make connection. Giving up.\n"));
    
    return connected ? fd : -1;
 #else
@@ -312,7 +340,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		  continue;
 	       }
 # endif
-	     fprintf(stderr, _("%s: Unknown host.\n"), host);
+	     print_error(_("%s: Unknown host.\n"), host);
 	     return -1;
 	  }
 	
@@ -358,7 +386,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 
 	this_host = (char *) inet_ntoa (s_in.sin_addr);
 	
-	if (TCP_Verbose_Reporting) fprintf (stderr, "trying %s\n", this_host);
+	if (TCP_Verbose_Reporting) print_error ("trying %s\n", this_host);
 	
 	not_connected = connect (s, (struct sockaddr *)&s_in, sizeof (s_in));
 	
@@ -371,7 +399,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
 		    continue;
 	       }
 # endif
-	     fprintf (stderr, _("connection to %s, port %d:"), 
+	     print_error (_("connection to %s, port %d:"), 
 		      (char *) this_host, port);
 	     perror ("");
 	  }
@@ -380,7 +408,7 @@ static int get_tcp_socket_1 (char *host, int port) /*{{{*/
    
    if (not_connected) 
      {
-	fprintf(stderr, _("Unable to make connection. Giving up.\n"));
+	print_error(_("Unable to make connection. Giving up.\n"));
 	(void) SLTCP_CLOSE (s);
 	return -1;
      }
@@ -483,17 +511,20 @@ static void dump_ssl_error_0 (void)
    int err;
    
    while (0 != (err = ERR_get_error()))
-     fprintf (stderr, "%s\n", ERR_error_string(err, 0));
+     print_error ("%s\n", ERR_error_string(err, 0));
 }
 
-unsigned long Fast_Random;
+#if !SLTCP_HAS_NSS_COMPAT
+static unsigned long Fast_Random;
 static unsigned long fast_random (void)
 {
    return (Fast_Random = Fast_Random * 69069U + 1013904243U);
 }
+#endif
 
 static int init_ssl_random (void)
 {
+#if !SLTCP_HAS_NSS_COMPAT
    time_t t;
    pid_t pid;
    unsigned int count;
@@ -515,10 +546,11 @@ static int init_ssl_random (void)
 	unsigned long r = fast_random ();
 	RAND_seed (&r, sizeof (unsigned long));
      }
+#endif				       /* !SLTCP_HAS_NSS_COMPAT */
    if (RAND_status ())
      return 0;
    
-   fprintf (stderr, _("Unable to generate enough entropy\n"));
+   print_error (_("Unable to generate enough entropy\n"));
    return -1;
 }
 
@@ -537,7 +569,7 @@ static SSL *alloc_ssl (void)
 	if (c == NULL)
 	  {
 	     dump_ssl_error_0 ();
-	     fprintf (stderr, _("SSL_CTX_new failed.\n"));
+	     print_error (_("SSL_CTX_new failed.\n"));
 	     return NULL;
 	  }
 	This_SSL_Ctx = c;
@@ -552,7 +584,7 @@ static SSL *alloc_ssl (void)
      return ssl;
    
    dump_ssl_error_0 ();
-   fprintf (stderr, _("SSL_new failed\n"));
+   print_error (_("SSL_new failed\n"));
    return NULL;
 }
 
@@ -569,17 +601,17 @@ static void dump_ssl_error (SSL *ssl, char *msg, int status)
       case SSL_ERROR_NONE:
 	break;
       case SSL_ERROR_ZERO_RETURN:
-	fprintf (stderr, _("Unexpected error: SSL connection closed\n"));
+	print_error (_("Unexpected error: SSL connection closed\n"));
 	break;
       case SSL_ERROR_SYSCALL:
-	fprintf (stderr, _("System call failed: errno = %d\n"), errno);
+	print_error (_("System call failed: errno = %d\n"), errno);
 	break;
       case SSL_ERROR_SSL:
-	fprintf (stderr, _("Possible protocol error\n"));
+	print_error (_("Possible protocol error\n"));
 	break;
      }
    dump_ssl_error_0 ();
-   if (msg != NULL) fprintf (stderr, _("%s failed\n"), msg);
+   if (msg != NULL) print_error (_("%s failed\n"), msg);
 }
 #endif				       /* SLTCP_HAS_SSL_SUPPORT */
 
@@ -598,7 +630,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
 	if (NULL == (ssl = alloc_ssl ()))
 	  return NULL;
 #else
-	fprintf (stderr, _("\n\n*** This program does not support SSL\n"));
+	print_error (_("\n\n*** This program does not support SSL\n"));
 	return NULL;
 #endif
      }
@@ -606,7 +638,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    tcp = (SLTCP_Type *) SLMALLOC (sizeof (SLTCP_Type));
    if (tcp == NULL)
      {
-	fprintf (stderr, _("Memory Allocation Failure.\n"));
+	print_error (_("Memory Allocation Failure.\n"));
 #if SLTCP_HAS_SSL_SUPPORT
 	dealloc_ssl (ssl);
 #endif
@@ -638,7 +670,7 @@ SLTCP_Type *sltcp_open_connection (char *host, int port, int use_ssl) /*{{{*/
    if (1 != SSL_set_fd (ssl, fd))
      {
 	/* Yuk.  SSL_set_fd returns 0 upon failure */
-	fprintf (stderr, _("SSL_set_fd failed\n"));
+	print_error (_("SSL_set_fd failed\n"));
 	sltcp_close (tcp);
 	return NULL;
      }
@@ -1066,7 +1098,7 @@ int sltcp_vfprintf (SLTCP_Type *tcp, char *fmt, va_list ap) /*{{{*/
    if ((tcp == NULL) || (-1 == tcp->tcp_fd))
      return -1;
    
-   slrn_vsnprintf (buf, sizeof (buf), fmt, ap);
+   (void) SLvsnprintf (buf, sizeof (buf), fmt, ap);
    
    return sltcp_fputs (tcp, buf);
 }
@@ -1192,7 +1224,6 @@ int sltcp_get_fd (SLTCP_Type *tcp)
 }
 
 
-
 #if 0 && defined(USE_WINSOCK_SLTCP)
 /* Some winsock notes */
 
