@@ -1,7 +1,7 @@
 /* -*- mode: C; mode: fold; -*- */
 /* Local spool support for slrn added by Olly Betts <olly@mantis.co.uk> */
 /* Modified by Thomas Schultz:
- * Copyright (c) 2001-2004 Thomas Schultz <tststs@gmx.de>
+ * Copyright (c) 2001-2006 Thomas Schultz <tststs@gmx.de>
  */
 #include "config.h"
 #include "slrnfeat.h"
@@ -40,6 +40,7 @@
 #include "slrndir.h"
 #include "snprintf.h"
 #include "vfile.h"
+#include "strutil.h"
 
 #ifndef SLRN_SPOOL_ROOT
 # define SLRN_SPOOL_ROOT "/var/spool/news" /* a common place for the newsspool */
@@ -65,17 +66,17 @@ char *Slrn_Overviewfmt_File;
 char *Slrn_Requests_File;
 
 static int spool_put_server_cmd (char *, char *, unsigned int );
-static int spool_select_group (char *, int *, int *);
+static int spool_select_group (char *, NNTP_Artnum_Type *, NNTP_Artnum_Type *);
 static int spool_refresh_groups (Slrn_Group_Range_Type *, int);
 static void spool_close_server (void);
 static int spool_has_cmd (char *);
 static int spool_initialize_server (void);
 static int spool_read_line (char *, unsigned int );
-static int spool_xpat_cmd (char *, int, int, char *);
-static int spool_select_article (int, char *);
-static int spool_get_article_size (int);
-static int spool_one_xhdr_command (char *, int, char *, unsigned int);
-static int spool_xhdr_command (char *, int, int);
+static int spool_xpat_cmd (char *, NNTP_Artnum_Type, NNTP_Artnum_Type, char *);
+static int spool_select_article (NNTP_Artnum_Type, char *);
+static int spool_get_article_size (NNTP_Artnum_Type);
+static int spool_one_xhdr_command (char *, NNTP_Artnum_Type, char *, unsigned int);
+static int spool_xhdr_command (char *, NNTP_Artnum_Type, NNTP_Artnum_Type);
 static int spool_list (char *);
 static int spool_list_newsgroups (void);
 static int spool_list_active (char *);
@@ -83,7 +84,7 @@ static int spool_send_authinfo (void);
 static int spool_read_xover (char *, unsigned int);
 static int spool_read_xpat (char *, unsigned int);
 static int spool_read_xhdr (char *, unsigned int);
-static int spool_article_num_exists (int);
+static int spool_article_num_exists (NNTP_Artnum_Type);
 static unsigned int spool_get_bytes (int clear);
 
 static Slrn_Server_Obj_Type Spool_Server_Obj;
@@ -96,11 +97,11 @@ static char *Spool_Group=NULL;
 static char *Spool_Group_Name;
 
 static FILE *Spool_fh_nov=NULL; /* we use the overview file lots, so keep it open */
-static int Spool_cur_artnum = 0;
+static NNTP_Artnum_Type Spool_cur_artnum = 0;
 
 /* These are set when the group is selected. */
-static int Spool_Max_Artnum = 0;
-static int Spool_Min_Artnum = 0;
+static NNTP_Artnum_Type Spool_Max_Artnum = 0;
+static NNTP_Artnum_Type Spool_Min_Artnum = 0;
 
 static int Spool_Doing_XOver;	       /* if non-zero, reading from ,overview */
 static int Spool_Doing_XPat;	       /* reading xpat */
@@ -116,6 +117,8 @@ static unsigned int bytes_read=0;
 
 /* Note: Please leave debugging output untranslated.
  *       It cannot be interpreted by the end user anyway. */
+static void debug_output (char *file, int line, char *fmt, ...) ATTRIBUTE_PRINTF(3,4);
+
 static void debug_output (char *file, int line, char *fmt, ...)
 {
    va_list ap;
@@ -158,6 +161,10 @@ static int spool_fclose_local (void)
 
    Spool_fhead=0;
    Spool_fFakingActive=0;
+#if SLANG_VERSION >= 20000
+   if (Spool_Output_Regexp != NULL)
+     SLregexp_free (Spool_Output_Regexp);
+#endif
    Spool_Output_Regexp = NULL;
    Spool_Ignore_Comments=0;
 
@@ -186,7 +193,7 @@ static FILE *spool_open_nov_file (void)
    return fp;
 }
 
-static int overview_file_seek (long fp, int cur, int dest)
+static int overview_file_seek (long fp, NNTP_Artnum_Type cur, NNTP_Artnum_Type dest)
 {
    int ch;
    
@@ -197,7 +204,7 @@ static int overview_file_seek (long fp, int cur, int dest)
              debug_output (__FILE__, __LINE__, "ftell returned -1; errno %d (%s).", errno, strerror(errno));
              return -1;
           }
-	if (1 != fscanf (Spool_fh_local, "%d", &cur))
+	if (1 != fscanf (Spool_fh_local, NNTP_FMT_ARTNUM, &cur))
 	  {
 	     spool_fclose_local ();
 	     return -1;
@@ -215,13 +222,13 @@ static int overview_file_seek (long fp, int cur, int dest)
    return 0;
 }
 
-static int Spool_XOver_Next;
-static int Spool_XOver_Max;
-static int Spool_XOver_Min;
+static NNTP_Artnum_Type Spool_XOver_Next;
+static NNTP_Artnum_Type Spool_XOver_Max;
+static NNTP_Artnum_Type Spool_XOver_Min;
 
-static int spool_nntp_xover (int min, int max)
+static int spool_nntp_xover (NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   int i, ch;
+   NNTP_Artnum_Type i;
    long fp;
 
    Spool_Doing_XOver = 0;
@@ -249,7 +256,7 @@ static int spool_nntp_xover (int min, int max)
              return ERR_FAULT;
           }
 
-	if ((1 != fscanf (Spool_fh_local,"%d", &i))
+	if ((1 != fscanf (Spool_fh_local, NNTP_FMT_ARTNUM, &i))
 	    || (i > min))
 	  {
 	     /* looks like we're after the start of the range */
@@ -260,6 +267,8 @@ static int spool_nntp_xover (int min, int max)
 	  }
 	else
 	  {
+	     int ch;
+
 	     while (((ch = getc(Spool_fh_local)) != '\n')
 		    && (ch != EOF))
 	       ; /* do nothing */
@@ -316,7 +325,7 @@ static int spool_read_xover (char *the_buf, unsigned int len)
 	  the_buf [buflen - 1] = 0;
    
 	/* check if we've reached the end of the requested range */
-	Spool_XOver_Next = atoi (the_buf);
+	Spool_XOver_Next = NNTP_STR_TO_ARTNUM (the_buf);
 	if (Spool_XOver_Next > Spool_XOver_Max)
 	  {
              if (-1 == fseek(Spool_fh_nov, pos, SEEK_SET))
@@ -350,16 +359,16 @@ static int spool_read_xover (char *the_buf, unsigned int len)
    return 1;
 }
 
-static int spool_find_artnum_from_msgid (char *msgid)
+static int spool_find_artnum_from_msgid (char *msgid, NNTP_Artnum_Type *idp)
 {
    char buf [4096];
    char *p;
-   int n;
    
    debug_output (NULL, -1, "spool_find_artnum_from_msgid('%s')", msgid);
 
    if (Slrn_Server_Obj->sv_has_xover == 0)
      {
+	NNTP_Artnum_Type n;
 	unsigned int len = strlen (msgid);
    
 	for (n = Spool_Min_Artnum; n <= Spool_Max_Artnum; n++)
@@ -369,13 +378,16 @@ static int spool_find_artnum_from_msgid (char *msgid)
 	     
 	     p = slrn_skip_whitespace (buf);
 	     if (0 == strncmp (p, msgid, len))
-	       return n;
+	       {
+		  *idp = n;
+		  return 0;
+	       }
 	  }
    
 	return -1;
      }
    
-   if (OK_XOVER != spool_nntp_xover (1, INT_MAX))
+   if (OK_XOVER != spool_nntp_xover (1, NNTP_ARTNUM_TYPE_MAX))
      return -1;
 
    while (1 == spool_read_xover (buf, sizeof(buf)))
@@ -384,20 +396,22 @@ static int spool_find_artnum_from_msgid (char *msgid)
 
 	/* 5th field is message id. */
 
-	if (NULL == (p = slrn_strchr(buf, '\t'))) continue;
-	if (NULL == (p = slrn_strchr(p + 1, '\t'))) continue;
-	if (NULL == (p = slrn_strchr(p + 1, '\t'))) continue;
-	if (NULL == (p = slrn_strchr(p + 1, '\t'))) continue;
+	if (NULL == (p = slrn_strbyte(buf, '\t'))) continue;
+	if (NULL == (p = slrn_strbyte(p + 1, '\t'))) continue;
+	if (NULL == (p = slrn_strbyte(p + 1, '\t'))) continue;
+	if (NULL == (p = slrn_strbyte(p + 1, '\t'))) continue;
 
 	p++; /* skip tab */
-	q = slrn_strchr(p,'\t');
+	q = slrn_strbyte(p,'\t');
 	if (q != NULL) *q='\0';
 
 	if (0 == strcmp(msgid, p))
 	  {
-	     debug_output (NULL, -1, "spool_find_artnum_from_msgid() returns %d",atoi(buf));
+	     debug_output (NULL, -1, ("spool_find_artnum_from_msgid() returns " NNTP_FMT_ARTNUM)
+			   ,NNTP_STR_TO_ARTNUM(buf));
 	     Spool_Doing_XOver = 0;
-	     return atoi(buf);
+	     *idp = NNTP_STR_TO_ARTNUM(buf);
+	     return 0;
 	  }
      }
    
@@ -408,20 +422,20 @@ static int spool_find_artnum_from_msgid (char *msgid)
 }
 
 
-static FILE *spool_open_article_num (int num)
+static FILE *spool_open_article_num (NNTP_Artnum_Type num)
 {
    char buf [SLRN_MAX_PATH_LEN];
    
-   slrn_snprintf (buf, sizeof (buf), "%s/%d", Spool_Group, num);
+   slrn_snprintf (buf, sizeof (buf), ("%s/" NNTP_FMT_ARTNUM), Spool_Group, num);
    
    return fopen (buf,"r");
 }
 
-static int spool_article_num_exists (int num)
+static int spool_article_num_exists (NNTP_Artnum_Type num)
 {
    char buf [SLRN_MAX_PATH_LEN];
    
-   slrn_snprintf (buf, sizeof (buf), "%s/%d", Spool_Group, num);
+   slrn_snprintf (buf, sizeof (buf), ("%s/" NNTP_FMT_ARTNUM), Spool_Group, num);
 
    if (1 == slrn_file_exists (buf))
      return 0;
@@ -429,11 +443,11 @@ static int spool_article_num_exists (int num)
    return -1;
 }
 
-static int spool_get_article_size (int num)
+static int spool_get_article_size (NNTP_Artnum_Type num)
 {
    char buf [SLRN_MAX_PATH_LEN];
    
-   slrn_snprintf (buf, sizeof (buf), "%s/%d", Spool_Group, num);
+   slrn_snprintf (buf, sizeof (buf), ("%s/" NNTP_FMT_ARTNUM), Spool_Group, num);
    
    return slrn_file_size (buf);
 }
@@ -454,14 +468,19 @@ static int spool_is_name_all_digits (char *p)
 
 /*{{{ The routines in this fold implement the sv_put_server_cmd */
 
-static int spool_nntp_head (int id, char *msgid, int *real_id)
+static int spool_nntp_head (NNTP_Artnum_Type id, char *msgid, NNTP_Artnum_Type *real_id)
 {
    spool_fclose_local();
 
    if (id == -1)
      {
-	if (msgid == NULL) id = Spool_cur_artnum;
-	else id = spool_find_artnum_from_msgid (msgid);
+	if (msgid == NULL) 
+	  id = Spool_cur_artnum;
+	else
+	  {
+	     if (-1 == spool_find_artnum_from_msgid (msgid, &id))
+	       id = -1;
+	  }
      }
 
    if (real_id != NULL) *real_id = id;
@@ -476,9 +495,9 @@ static int spool_nntp_head (int id, char *msgid, int *real_id)
    return OK_HEAD; /* Head follows */
 }
 
-static int spool_nntp_next (int *id)
+static int spool_nntp_next (NNTP_Artnum_Type *id)
 {
-   int i;
+   NNTP_Artnum_Type i;
    
    spool_fclose_local();
 
@@ -490,14 +509,15 @@ static int spool_nntp_next (int *id)
 	     Spool_cur_artnum = i;
 	     if (id != NULL) *id = i;
 	     
-	     debug_output (NULL, -1, "NEXT found article %d",Spool_cur_artnum);
+	     debug_output (NULL, -1, ("NEXT found article " NNTP_FMT_ARTNUM), Spool_cur_artnum);
 
 	     return OK_NOTEXT; /* No text sent -- stat, next, last */
 	  }
      }
 
-   debug_output (NULL, -1, "No NEXT -- %d > %d", Spool_cur_artnum, Spool_Max_Artnum);
-   
+   debug_output (NULL, -1, ("No NEXT -- " NNTP_FMT_ARTNUM " > " NNTP_FMT_ARTNUM), 
+		 Spool_cur_artnum, Spool_Max_Artnum);
+
    return ERR_NONEXT; /* No next article in this group */
 }
 
@@ -564,14 +584,15 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 
    if (0 == fGMT)
      {
-	struct tm *t;
 #ifdef HAVE_TIMEZONE
+	struct tm *t;
 	(void) localtime (&threshold); /* This call sets timezone */
 	threshold += timezone;
 	t = localtime (&threshold);
 	if (t->tm_isdst) threshold -= (time_t) 3600;
 #else
 # ifdef HAVE_TM_GMTOFF
+	struct tm *t;
 	time_t tt;
 	
 	t = localtime (&threshold);
@@ -583,7 +604,7 @@ static int spool_nntp_newgroups (char *line, char *buf, unsigned int len)
 #endif /* NOT HAVE_TIMEZONE */
      }
 
-   debug_output (NULL, -1, "threshold for spool_nntp_newsgrups is %d",threshold);
+   debug_output (NULL, -1, "threshold for spool_nntp_newsgrups is %lu", (unsigned long) threshold);
 
    spool_fclose_local();
    Spool_fh_local = fopen (Slrn_ActiveTimes_File, "r");
@@ -722,8 +743,8 @@ static int spool_put_server_cmd (char *line, char *buf, unsigned int len)
    nntpmap = Spool_NNTP_Maps;
    while (nntpmap->name != NULL)
      {
-	if (!slrn_case_strncmp ((unsigned char *)nntpmap->name,
-				(unsigned char *) line, nntpmap->len))
+	if (!slrn_case_strncmp (nntpmap->name,
+				 line, nntpmap->len))
 	  return (*nntpmap->f)(line, buf, len);
 
 	nntpmap++;
@@ -736,13 +757,13 @@ static int spool_put_server_cmd (char *line, char *buf, unsigned int len)
 /*}}}*/
 
 
-static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, int *min, int *max)
+static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, NNTP_Artnum_Type *min, NNTP_Artnum_Type *max)
 {
    Slrn_Dirent_Type *ep;
    char *p;
-   long l;
-   long hi = 0;
-   long lo = LONG_MAX;
+   NNTP_Artnum_Type l;
+   NNTP_Artnum_Type hi = 0;
+   NNTP_Artnum_Type lo = NNTP_ARTNUM_TYPE_MAX;
 
    /* Scan through all the files, checking the ones with numbers for names */
    while ((ep = slrn_read_dir(dp)) != NULL)
@@ -753,7 +774,7 @@ static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, int *min, int *max)
 	if (0 == spool_is_name_all_digits (p))
 	  continue;
 
-	if (0 == (l = atol (p)))
+	if (0 == (l = NNTP_STR_TO_ARTNUM(p)))
 	  continue;
 
 	if (l < lo)
@@ -772,11 +793,11 @@ static int spool_read_minmax_from_dp (Slrn_Dir_Type *dp, int *min, int *max)
    return 0;
 }
 
-static int spool_read_minmax_file (int *min, int *max, char *group_dir)
+static int spool_read_minmax_file (NNTP_Artnum_Type *min, NNTP_Artnum_Type *max, char *group_dir)
 {
    char *file;
    FILE *fp;
-   char buf[256];
+   char buf[512];
    int status;
 
    file = slrn_spool_dircat (group_dir, ".minmax", 0);
@@ -792,7 +813,7 @@ static int spool_read_minmax_file (int *min, int *max, char *group_dir)
 
    status = 0;
    if ((NULL == fgets (buf, sizeof (buf), fp))
-       || (2 != sscanf (buf, "%d %d", min, max)))
+       || (2 != sscanf (buf, NNTP_FMT_ARTNUM_2, min, max)))
      status = -1;
    
    fclose (fp);
@@ -805,7 +826,7 @@ static int spool_read_minmax_file (int *min, int *max, char *group_dir)
  * or looking at the files in the directory.
  * Returns 0 on success, -1 on failure
  */
-static int spool_read_minmax_from_dir (int *min, int *max, char *dir)
+static int spool_read_minmax_from_dir (NNTP_Artnum_Type *min, NNTP_Artnum_Type *max, char *dir)
 {
    Slrn_Dir_Type *dp;
 
@@ -836,7 +857,7 @@ static int spool_read_minmax_from_dir (int *min, int *max, char *dir)
  * Returns 0 on success, -1 on failure
  * (failure => active file didn't open, or the group wasn't in it)
  */
-static int spool_read_minmax_from_active( char *name, int *min, int *max )
+static int spool_read_minmax_from_active( char *name, NNTP_Artnum_Type *min, NNTP_Artnum_Type *max )
 {
    char buf[512];
    unsigned int len;
@@ -855,11 +876,11 @@ static int spool_read_minmax_from_active( char *name, int *min, int *max )
 	    && (0 == memcmp (buf, name, len)))
 	  {
 	     spool_fclose_local ();
-	     if (2 != sscanf (buf + len + 1, "%d%d", max, min))
+	     if (2 != sscanf (buf + len + 1, NNTP_FMT_ARTNUM NNTP_FMT_ARTNUM, max, min))
 	       return -1;
 
 	     Spool_Max_Artnum = *max;
-	     debug_output (NULL, -1, "from active:%s %d %d", name, *min, *max);
+	     debug_output (NULL, -1, "from active:%s " NNTP_FMT_ARTNUM_2, name, *min, *max);
 	     return 0;
 	  }
 	buf[len] = 0;
@@ -873,7 +894,7 @@ static int spool_read_minmax_from_active( char *name, int *min, int *max )
 /* Get the lowest and highest article numbers from the overview file
  * Returns 0 on success, -1 on failure
  */
-static int spool_read_minmax_from_overview (char *name, int *min, int *max)
+static int spool_read_minmax_from_overview (char *name, NNTP_Artnum_Type *min, NNTP_Artnum_Type *max)
 {
    /* chunk size to step back through .overview files by
     * when trying to find start of last line */
@@ -892,7 +913,7 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
    else
      /* !HACK! this assumes the overview file is rewound */
      Spool_Server_Obj.sv_has_xover = ((Spool_fh_nov != NULL)
-				      && (1 == fscanf (Spool_fh_nov,"%d", min)));
+				      && (1 == fscanf (Spool_fh_nov, NNTP_FMT_ARTNUM, min)));
 
    if (0 == Spool_Server_Obj.sv_has_xover)
      return -1;
@@ -951,7 +972,7 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
         return -1;
      }
 
-   if (1 != fscanf (Spool_fh_nov, "%d", max))
+   if (1 != fscanf (Spool_fh_nov, NNTP_FMT_ARTNUM, max))
      {
         debug_output (__FILE__, __LINE__, "Hmmm, unable to understand overview file - no integer found?");
         return -1;
@@ -959,11 +980,11 @@ static int spool_read_minmax_from_overview (char *name, int *min, int *max)
 
    rewind (Spool_fh_nov);
 
-   debug_output (NULL, -1, "%s %d %d",name,*min,*max);
+   debug_output (NULL, -1, ("%s " NNTP_FMT_ARTNUM_2), name,*min,*max);
    return 0;
 }
 
-static int spool_select_group (char *name, int *min, int *max)
+static int spool_select_group (char *name, NNTP_Artnum_Type *min, NNTP_Artnum_Type *max)
 {
    /* close any open files */
    spool_fclose_local();
@@ -992,7 +1013,7 @@ static int spool_select_group (char *name, int *min, int *max)
    Spool_Max_Artnum = *max;
    Spool_Min_Artnum = *min;
 
-   debug_output (NULL, -1, "Group: %s %d - %d", name, *min, *max);
+   debug_output (NULL, -1, "Group: %s " NNTP_FMT_ARTRANGE, name, *min, *max);
    return OK_GROUP;
 }
 
@@ -1094,19 +1115,15 @@ static int spool_read_line (char *line, unsigned int len)
 	char buf[NNTP_BUFFER_SIZE];
 	int retval = -1;
 	
-	if (buf == NULL) return -1;
+	/* if (buf == NULL) return -1; */
 	if ((len > 33) &&
 	    (1 == (retval = spool_read_xhdr (buf, sizeof (buf)))))
 	  {
-	     int numlen;
-#ifdef HAVE_ANSI_SPRINTF
-	     numlen =
-#endif
-	     sprintf (line, "%d ", (Spool_XOver_Next - 1)); /* safe */
-#ifndef HAVE_ANSI_SPRINTF
+	     unsigned int numlen;
+	     (void) slrn_snprintf (line, len, (NNTP_FMT_ARTNUM " "), (Spool_XOver_Next - 1));
 	     numlen = strlen (line);
-#endif
-	     slrn_strncpy (line + numlen, buf, len - numlen);
+	     if (len > numlen)
+	       slrn_strncpy (line + numlen, buf, len - numlen);
 	  }
 	return retval;
      }
@@ -1119,7 +1136,7 @@ static int spool_read_line (char *line, unsigned int len)
 typedef struct
 {
    int xover_field;
-   unsigned int rmin, rmax;
+   NNTP_Artnum_Type rmin, rmax;
    char header[80];
    char pat[256];
 }
@@ -1144,11 +1161,11 @@ static int spool_read_xpat (char *buf, unsigned int len)
    
    if (Spool_XPat_Struct.xover_field == -1)
      {
-	unsigned int num;
+	NNTP_Artnum_Type num;
 	
 	for (num = Spool_XPat_Struct.rmin; num <= Spool_XPat_Struct.rmax; num++)
 	  {
-	     if (-1 == spool_one_xhdr_command (Spool_XPat_Struct.header, (int) num,
+	     if (-1 == spool_one_xhdr_command (Spool_XPat_Struct.header, num,
 					       tmpbuf, sizeof (tmpbuf)))
 	       continue;
 	     
@@ -1159,7 +1176,7 @@ static int spool_read_xpat (char *buf, unsigned int len)
 		  Spool_Doing_XPat = 1;
 		  Spool_XPat_Struct.rmin = num + 1;
 		  
-		  slrn_snprintf (buf, len, "%d ", num);
+		  slrn_snprintf (buf, len, (NNTP_FMT_ARTNUM " "), num);
 		  blen = strlen (buf);
 		  
 		  strncpy (buf + blen, tmpbuf, len - blen);
@@ -1176,12 +1193,12 @@ static int spool_read_xpat (char *buf, unsigned int len)
 	/* read the overview file until the right field matches the pattern */
 	while (NULL != (fgets (tmpbuf, sizeof (tmpbuf), Spool_fh_local)))
 	  {
-	     int field = Spool_XPat_Struct.xover_field + 1;
 	     /* first field is article number; "+1" skips it */
-	     unsigned int num = (unsigned int) atoi (tmpbuf);
+	     int field = Spool_XPat_Struct.xover_field + 1;
+	     NNTP_Artnum_Type num = NNTP_STR_TO_ARTNUM (tmpbuf);
 	     char *b = tmpbuf, *end;
 	     
-	     if (num > Spool_XPat_Struct.rmax)
+	     if ((num > Spool_XPat_Struct.rmax) || (num < 0))
 	       {
 		  spool_fclose_local ();
 		  return 0;
@@ -1189,7 +1206,7 @@ static int spool_read_xpat (char *buf, unsigned int len)
 	     
 	     while (field)
 	       {
-		  b = strchr (b, '\t');
+		  b = slrn_strbyte (b, '\t');
 		  /* If we reach the end of the overview line before finding
 		   * the correct field, the overview file must be corrupt. */
 		  if (b == NULL)
@@ -1203,7 +1220,7 @@ static int spool_read_xpat (char *buf, unsigned int len)
 		  b++;
 		  field--;
 	       }
-	     end = strchr (b, '\t');
+	     end = slrn_strbyte (b, '\t');
 	     if (end != NULL)
 	       *end = '\0';
 	     
@@ -1212,9 +1229,9 @@ static int spool_read_xpat (char *buf, unsigned int len)
 		  unsigned int blen;
 		  
 		  Spool_Doing_XPat = 1;
-		  slrn_snprintf (buf, len, "%u ", num);
+		  slrn_snprintf (buf, len, (NNTP_FMT_ARTNUM " "), num);
 		  blen = strlen (buf);
-		  
+
 		  strncpy (buf + blen, b, len - blen);
 		  buf[len - 1] = 0;
 		  return 1;
@@ -1226,7 +1243,7 @@ static int spool_read_xpat (char *buf, unsigned int len)
    return 0;
 }
 
-static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
+static int spool_xpat_cmd (char *hdr, NNTP_Artnum_Type rmin, NNTP_Artnum_Type rmax, char *pat)
 {
    static char *overview_headers [] = 
      {
@@ -1262,7 +1279,7 @@ static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
 	     char *h = overview_headers [field];
 	     
 	     if (h == NULL) break;
-	     if (0 == slrn_case_strcmp ((unsigned char *) h, (unsigned char *) hdr))
+	     if (0 == slrn_case_strcmp ( h,  hdr))
 	       {
 		  Spool_XPat_Struct.xover_field = field;
 		  break;
@@ -1285,7 +1302,7 @@ static int spool_xpat_cmd (char *hdr, int rmin, int rmax, char *pat)
    return OK_HEAD;
 }
 
-static int spool_select_article (int n, char *msgid)
+static int spool_select_article (NNTP_Artnum_Type n, char *msgid)
 {
    /*    printf("spool_select_article(%d,%s)\n",n,msgid); */
 
@@ -1294,7 +1311,7 @@ static int spool_select_article (int n, char *msgid)
 	if ((msgid == NULL) || (*msgid == 0))
 	  return -1;
 	
-	if (-1 == (n = spool_find_artnum_from_msgid (msgid)))
+	if (-1 == spool_find_artnum_from_msgid (msgid, &n))
 	  return ERR_NOARTIG;
      }
 
@@ -1309,7 +1326,7 @@ static int spool_select_article (int n, char *msgid)
 
 
 /* The hdr string should NOT include the ':' */
-static int spool_one_xhdr_command (char *hdr, int num, char *buf,
+static int spool_one_xhdr_command (char *hdr, NNTP_Artnum_Type num, char *buf,
 				   unsigned int buflen)
 {
    char tmpbuf [1024];
@@ -1327,7 +1344,7 @@ static int spool_one_xhdr_command (char *hdr, int num, char *buf,
    while (1 == spool_read_fhlocal (tmpbuf, sizeof (tmpbuf)))
      {
 	char *b;
-	if (slrn_case_strncmp ((unsigned char *) tmpbuf, (unsigned char *) hdr, colon)
+	if (slrn_case_strncmp ( tmpbuf,  hdr, colon)
 	    || (tmpbuf[colon] != ':'))
 	  continue;
 
@@ -1347,10 +1364,11 @@ static int spool_one_xhdr_command (char *hdr, int num, char *buf,
    return -1;
 }
 
-static int spool_xhdr_command (char *field, int min, int max)
+static int spool_xhdr_command (char *field, NNTP_Artnum_Type min, NNTP_Artnum_Type max)
 {
-   debug_output (NULL, -1, "spool_xhdr_command(%s,%d,%d)", field, min, max);
-   
+   debug_output (NULL, -1, ("spool_xhdr_command(%s " NNTP_FMT_ARTNUM_2),
+		 field, min, max);
+
    spool_fclose_local ();
    
    if (max > Spool_Max_Artnum)
@@ -1391,7 +1409,7 @@ static int spool_read_xhdr (char *the_buf, unsigned int len)
 
 static int spool_list (char *what)
 {
-   if (!slrn_case_strcmp ((unsigned char*)what,(unsigned char*)"overview.fmt"))
+   if (!slrn_case_strcmp (what,"overview.fmt"))
      {
 	spool_fclose_local();
 	Spool_fh_local=fopen(Slrn_Overviewfmt_File,"r");
@@ -1424,6 +1442,7 @@ static int spool_list_newsgroups (void)
 
 /* Having an independant buffer for the spool code might spare us mysterious
  * bugs, so I'm not simply using slrn_compile_regexp_pattern */
+#if SLANG_VERSION < 20000
 static SLRegexp_Type *spool_compile_regexp_pattern (char *pat)
 {
    static unsigned char compiled_pattern_buf [512];
@@ -1441,13 +1460,23 @@ static SLRegexp_Type *spool_compile_regexp_pattern (char *pat)
      }
    return &re;
 }
+#endif
 
 static int spool_list_active (char *pat)
 {
    spool_fclose_local();
    Spool_fh_local=fopen (Slrn_Active_File,"r");
    if (pat != NULL)
-     Spool_Output_Regexp=spool_compile_regexp_pattern (slrn_fix_regexp (pat));
+     {
+#if SLANG_VERSION < 20000
+	Spool_Output_Regexp=spool_compile_regexp_pattern (slrn_fix_regexp (pat));
+#else
+	if (Spool_Output_Regexp != NULL)
+	  SLregexp_free (Spool_Output_Regexp);
+	Spool_Output_Regexp = SLregexp_compile (slrn_fix_regexp(pat),0);
+#endif
+     }
+	
    if (!Spool_fh_local)
      {
 	spool_fake_active(Slrn_Spool_Root);
@@ -1757,7 +1786,10 @@ Slrn_Range_Type *slrn_spool_get_no_body_ranges (char *group)
    
    if (NULL != (vline = vgets (vp, &vlen)))
      {
-	vline[vlen] = 0; /* make sure line is NULL terminated */
+	if (vline[vlen-1] == '\n')
+	  vline[vlen-1] = 0; /* make sure line is NULL terminated */
+	else
+	  vline[vlen] = 0;
 	retval = slrn_ranges_from_newsrc_line (vline);
      }
    
@@ -1787,8 +1819,11 @@ Slrn_Range_Type *slrn_spool_get_requested_ranges (char *group) /*{{{*/
 	if ((p == pmax) || (p == vline) ||
 	    (strncmp(vline, group, (p-vline))))
 	  continue;
-     
-	vline[vlen-1] = 0;	       /* kill \n and NULL terminate */
+
+	if (vline[vlen-1] == '\n')
+	  vline[vlen-1] = 0;
+	else
+	  vline[vlen] = 0;
 	
 	retval = slrn_ranges_from_newsrc_line (p+1);
 	break;
