@@ -1,7 +1,7 @@
 /*
  This file is part of SLRN.
 
- Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 1994, 1999, 2007-2008 John E. Davis <jed@jedsoft.org>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -23,11 +23,11 @@
 #include <stdarg.h>
 
 extern void slrn_tty_vmessage (FILE *, char *, va_list);
-extern void slrn_tty_error (char *, ...);
-extern void slrn_tty_message (char *, ...);
-extern int slrn_message (char *, ...);
-extern int slrn_message_now (char *, ...);
-extern void slrn_error (char *, ...);
-extern void slrn_error_now (unsigned int, char *, ...);
+extern void slrn_tty_error (char *, ...) ATTRIBUTE_PRINTF(1,2);
+extern void slrn_tty_message (char *, ...) ATTRIBUTE_PRINTF(1,2);
+extern int slrn_message (char *, ...) ATTRIBUTE_PRINTF(1,2);
+extern int slrn_message_now (char *, ...) ATTRIBUTE_PRINTF(1,2);
+extern void slrn_verror (char *, va_list);
+extern void slrn_error_now (unsigned int, char *, ...) ATTRIBUTE_PRINTF(2,3);
 
 #endif				       /* _SLRN_TTYMSG_H */
